# Copyright (c) 2016, GohighSec
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import ast
import re

from hacking import core


"""
Guidelines for writing new hacking checks

 - Use only for Barbican specific tests. OpenStack general tests
   should be submitted to the common 'hacking' module.
 - Pick numbers in the range B3xx. Find the current test with
   the highest allocated number and then pick the next value.
 - Keep the test method code in the source file ordered based
   on the B3xx value.
 - List the new rule in the top level HACKING.rst file
 - Add test cases for each new rule to barbican/tests/test_hacking.py
"""

oslo_namespace_imports = re.compile(r"from[\s]*oslo[.](.*)")
dict_constructor_with_list_copy_re = re.compile(r".*\bdict\((\[)?(\(|\[)")
no_log_warn = re.compile(r".*LOG.warn\(.*\)")


class BaseASTChecker(ast.NodeVisitor):
    """Provides a simple framework for writing AST-based checks.

    Subclasses should implement visit_* methods like any other AST visitor
    implementation. When they detect an error for a particular node the
    method should call ``self.add_error(offending_node)``. Details about
    where in the code the error occurred will be pulled from the node
    object.

    Subclasses should also provide a class variable named CHECK_DESC to
    be used for the human readable error message.

    """

    CHECK_DESC = 'No check message specified'

    def __init__(self, tree, filename):
        """This object is created automatically by pycodestyle.

        :param tree: an AST tree
        :param filename: name of the file being analyzed
                         (ignored by our checks)
        """
        self._tree = tree
        self._errors = []

    def run(self):
        """Called automatically by pycodestyle."""
        self.visit(self._tree)
        return self._errors

    def add_error(self, node, message=None):
        """Add an error caused by a node to the list of errors for pep8."""

        message = message or self.CHECK_DESC
        error = (node.lineno, node.col_offset, message, self.__class__)
        self._errors.append(error)

    def _check_call_names(self, call_node, names):
        if isinstance(call_node, ast.Call):
            if isinstance(call_node.func, ast.Name):
                if call_node.func.id in names:
                    return True
        return False


class CheckLoggingFormatArgs(BaseASTChecker):
    """Check for improper use of logging format arguments.

    LOG.debug("Volume %s caught fire and is at %d degrees C and climbing.",
              ('volume1', 500))

    The format arguments should not be a tuple as it is easy to miss.

    """
    name = "check_logging_format_args"
    version = "1.0"

    CHECK_DESC = 'B310 Log method arguments should not be a tuple.'
    LOG_METHODS = [
        'debug', 'info',
        'warn', 'warning',
        'error', 'exception',
        'critical', 'fatal',
        'trace', 'log'
    ]

    def _find_name(self, node):
        """Return the fully qualified name or a Name or Attribute."""
        if isinstance(node, ast.Name):
            return node.id
        elif (isinstance(node, ast.Attribute)
                and isinstance(node.value, (ast.Name, ast.Attribute))):
            method_name = node.attr
            obj_name = self._find_name(node.value)
            if obj_name is None:
                return None
            return obj_name + '.' + method_name
        elif isinstance(node, str):
            return node
        else:  # could be Subscript, Call or many more
            return None

    def visit_Call(self, node):
        """Look for the 'LOG.*' calls."""
        # extract the obj_name and method_name
        if isinstance(node.func, ast.Attribute):
            obj_name = self._find_name(node.func.value)
            if isinstance(node.func.value, ast.Name):
                method_name = node.func.attr
            elif isinstance(node.func.value, ast.Attribute):
                obj_name = self._find_name(node.func.value)
                method_name = node.func.attr
            else:  # could be Subscript, Call or many more
                return super(CheckLoggingFormatArgs, self).generic_visit(node)

            # obj must be a logger instance and method must be a log helper
            if (obj_name != 'LOG'
                    or method_name not in self.LOG_METHODS):
                return super(CheckLoggingFormatArgs, self).generic_visit(node)

            # the call must have arguments
            if not len(node.args):
                return super(CheckLoggingFormatArgs, self).generic_visit(node)

            # any argument should not be a tuple
            for arg in node.args:
                if isinstance(arg, ast.Tuple):
                    self.add_error(arg)

        return super(CheckLoggingFormatArgs, self).generic_visit(node)


@core.flake8ext
def check_oslo_namespace_imports(logical_line, filename, noqa):
    """'oslo_' should be used instead of 'oslo.'

    B317
    """
    if noqa:
        return
    if re.match(oslo_namespace_imports, logical_line):
        msg = ("B317: '%s' must be used instead of '%s'.") % (
            logical_line.replace('oslo.', 'oslo_'),
            logical_line)
        yield (0, msg)


@core.flake8ext
def dict_constructor_with_list_copy(logical_line):
    """Use a dict comprehension instead of a dict constructor

    B318
    """
    msg = (
        "B318: Must use a dict comprehension instead of a dict constructor "
        "with a sequence of key-value pairs."
    )
    if dict_constructor_with_list_copy_re.match(logical_line):
        yield (0, msg)


@core.flake8ext
def no_log_warn_check(logical_line):
    """Disallow 'LOG.warn'

    B320
    """
    msg = ("B320: LOG.warn is deprecated, please use LOG.warning!")
    if re.match(no_log_warn, logical_line):
        yield (0, msg)



# Copyright (c) 2015 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from unittest import mock

from barbican.api.middleware import context
from barbican.tests import utils


class TestUnauthenticatedContextMiddleware(utils.BaseTestCase):

    def setUp(self):
        super(TestUnauthenticatedContextMiddleware, self).setUp()
        self.app = mock.MagicMock()
        self.middleware = context.UnauthenticatedContextMiddleware(self.app)

    def test_role_defaults_to_admin(self):
        request = mock.MagicMock()
        request.headers = {'X-Project-Id': 'trace'}
        request.environ = {}

        with mock.patch('barbican.context.RequestContext') as rc:
            self.middleware.process_request(request)
            rc.assert_called_with(
                project_id='trace',
                is_admin=True,
                user_id=None,
                roles=['admin'],
                request_id=request.request_id,
                project_domain=None,
                domain=None,
                user_domain=None
            )

    def test_role_used_from_header(self):
        request = mock.MagicMock()
        request.headers = {'X-Project-Id': 'trace', 'X-Roles': 'something'}
        request.environ = {}

        with mock.patch('barbican.context.RequestContext') as rc:
            self.middleware.process_request(request)
            rc.assert_called_with(
                project_id='trace',
                is_admin=False,
                user_id=None,
                roles=['something'],
                request_id=request.request_id,
                project_domain=None,
                domain=None,
                user_domain=None
            )



# Copyright (c) 2015 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Retry/scheduler classes and logic.
"""
import random

from oslo_service import service
from oslo_utils import timeutils

from barbican.common import config
from barbican.common import utils
from barbican.model import models
from barbican.model import repositories
from barbican.queue import client as async_client

LOG = utils.getLogger(__name__)

CONF = config.CONF


def _compute_next_periodic_interval():
    periodic_interval = (
        CONF.retry_scheduler.periodic_interval_max_seconds
    )

    # Return +- 20% of interval.
    return random.uniform(0.8 * periodic_interval,  # nosec
                          1.2 * periodic_interval)


class PeriodicServer(service.Service):
    """Server to process retry and scheduled tasks.

    This server is an Oslo service. This server runs a thread to periodically
    check tasks that need to be retried, and then sends them up to the RPC
    queue for later processing by a worker node.
    """
    def __init__(self, queue_resource=None):
        super(PeriodicServer, self).__init__()

        # Setting up db engine to avoid lazy initialization
        repositories.setup_database_engine_and_factory()

        # Connect to the worker queue, to send retry RPC tasks to it later.
        self.queue = queue_resource or async_client.TaskClient()

        # Start the task retry periodic scheduler process up.
        periodic_interval = (
            CONF.retry_scheduler.periodic_interval_max_seconds
        )
        self.tg.add_dynamic_timer(
            self._check_retry_tasks,
            initial_delay=CONF.retry_scheduler.initial_delay_seconds,
            periodic_interval_max=periodic_interval)

        self.order_retry_repo = repositories.get_order_retry_tasks_repository()

    def start(self):
        LOG.info("Starting the PeriodicServer")
        super(PeriodicServer, self).start()

    def stop(self, graceful=True):
        LOG.info("Halting the PeriodicServer")
        super(PeriodicServer, self).stop(graceful=graceful)

    def _check_retry_tasks(self):
        """Periodically check to see if tasks need to be scheduled.

        :return: Return the number of seconds to wait before invoking this
            method again.
        """
        total_tasks_processed = 0
        try:
            total_tasks_processed = self._process_retry_tasks()
        except Exception:
            LOG.exception("Problem seen processing scheduled retry tasks")

        # Return the next delay before this method is invoked again.
        check_again_in_seconds = _compute_next_periodic_interval()
        LOG.info("Done processing '%(total)s' tasks, will check again in "
                 "'%(next)s' seconds.",
                 {
                     'total': total_tasks_processed,
                     'next': check_again_in_seconds
                 }
                 )
        return check_again_in_seconds

    def _process_retry_tasks(self):
        """Scan for and then re-queue tasks that are ready to retry."""
        LOG.info("Processing scheduled retry tasks:")

        # Retrieve tasks to retry.
        entities, total = self._retrieve_tasks()

        # Create RPC tasks for each retry task found.
        for task in entities:
            self._enqueue_task(task)

        return total

    def _retrieve_tasks(self):
        """Retrieve a list of tasks to retry."""
        repositories.start()
        try:
            entities, _, _, total = self.order_retry_repo.get_by_create_date(
                only_at_or_before_this_date=timeutils.utcnow(),
                suppress_exception=True)
        finally:
            repositories.clear()

        return entities, total

    def _enqueue_task(self, task):
        """Re-enqueue the specified task."""
        retry_task_name = 'N/A'
        retry_args = 'N/A'
        retry_kwargs = 'N/A'

        # Start a new isolated database transaction just for this task.
        repositories.start()
        try:
            # Invoke queue client to place retried RPC task on queue.
            retry_task_name = task.retry_task
            retry_args = task.retry_args
            retry_kwargs = task.retry_kwargs
            retry_method = getattr(self.queue, retry_task_name)
            retry_method(*retry_args, **retry_kwargs)

            # Remove the retry record from the queue.
            task.status = models.States.ACTIVE
            self.order_retry_repo.delete_entity_by_id(task.id, None)

            repositories.commit()

            LOG.debug(
                "(Enqueued method '{0}' with args '{1}' and "
                "kwargs '{2}')".format(
                    retry_task_name, retry_args, retry_kwargs))
        except Exception:
            LOG.exception("Problem enqueuing method '%(name)s' with args "
                          "'%(args)s' and kwargs '%(kwargs)s'.",
                          {
                              'name': retry_task_name,
                              'args': retry_args,
                              'kwargs': retry_kwargs
                          }
                          )
            repositories.rollback()
        finally:
            repositories.clear()



#!/usr/bin/env python3

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.
import argparse
import sys

from barbican.plugin.crypto import pkcs11


class KeyGenerator(object):

    def __init__(self, ffi=None):
        self.parser = self.get_main_parser()
        self.subparsers = self.parser.add_subparsers(
            title='subcommands',
            description='Action to perform'
        )
        self.add_mkek_args()
        self.add_hmac_args()
        self.args = self.parser.parse_args()
        if not self.args.passphrase:
            password = input("Please enter your password: ")
        self.pkcs11 = pkcs11.PKCS11(
            library_path=self.args.library_path,
            login_passphrase=self.args.passphrase or password,
            rw_session=True,
            slot_id=int(self.args.slot_id),
            ffi=ffi,
            encryption_mechanism='CKM_AES_CBC'
        )
        self.session = self.pkcs11.get_session()

    def get_main_parser(self):
        """Create a top-level parser and arguments."""
        parser = argparse.ArgumentParser(
            description='Barbican MKEK & HMAC Generator'
        )
        parser.add_argument(
            '--library-path',
            default='/usr/lib/libCryptoki2_64.so',
            help='Path to vendor PKCS11 library'
        )
        parser.add_argument(
            '--passphrase',
            default=None,
            help='Password to login to PKCS11 session'
        )
        parser.add_argument(
            '--slot-id',
            default=1,
            help='HSM Slot id (Should correspond to a configured PKCS11 slot)'
        )

        return parser

    def add_mkek_args(self):
        """Create MKEK generation parser and arguments."""
        create_parser = self.subparsers.add_parser('mkek', help='Generates a '
                                                   'new MKEK.')
        create_parser.add_argument('--length', '-l', default=32,
                                   help='the length of the MKEK')
        create_parser.add_argument('--label', '-L', default='primarymkek',
                                   help='the label for the MKEK')
        create_parser.set_defaults(func=self.generate_mkek)

    def add_hmac_args(self):
        """Create HMAC generation parser and arguments."""
        create_parser = self.subparsers.add_parser('hmac', help='Generates a '
                                                   'new HMAC.')
        create_parser.add_argument('--type', '-t', default='CKK_AES',
                                   help='HMAC key type, one of: '
                                   '\'CKK_AES\', \'CKK_GENERIC_SECRET\' or '
                                   '\'CKK_SHA256_HMAC\'')
        create_parser.add_argument('--keygen', '-g',
                                   default='CKM_AES_KEY_GEN',
                                   help='HMAC key generation mechanism, '
                                   'one of: \'CKM_AES_KEY_GEN\' '
                                   '\'CKM_NC_SHA256_HMAC_KEY_GEN\' or '
                                   '\'CKM_GENERIC_SECRET_KEY_GEN\'')
        create_parser.add_argument('--length', '-l', default=32,
                                   help='the length of the HMACKEY')
        create_parser.add_argument('--label', '-L', default='primaryhmac',
                                   help='the label for the HMAC')
        create_parser.set_defaults(func=self.generate_hmac)

    def verify_label_does_not_exist(self, key_type, label, session):
        key_handle = self.pkcs11.get_key_handle(key_type, label, session)
        if key_handle:
            print(
                "The label {label} already exists! "
                "Please try again.".format(label=label)
            )
            sys.exit(1)

    def generate_mkek(self, args):
        """Process the generate MKEK with given arguments"""
        self.verify_label_does_not_exist(args.type, args.label, self.session)
        self.pkcs11.generate_key('CKK_AES', int(args.length),
                                 'CKM_AES_KEY_GEN',
                                 self.session, args.label,
                                 encrypt=True, wrap=True, master_key=True)
        print("MKEK successfully generated!")

    def generate_hmac(self, args):
        """Process the generate HMAC with given arguments"""
        self.verify_label_does_not_exist(args.type, args.label, self.session)
        self.pkcs11.generate_key(args.type, int(args.length), args.keygen,
                                 self.session, args.label,
                                 sign=True, master_key=True)
        print("HMAC successfully generated!")

    def execute(self):
        """Parse the command line arguments."""
        try:
            self.args.func(self.args)
        except Exception as e:
            print(e)
        finally:
            self.pkcs11.return_session(self.session)


def main():
    kg = KeyGenerator()
    kg.execute()


if __name__ == '__main__':
    main()



"""
Copyright 2014-2015 Rackspace

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from functionaltests.api.v1.behaviors import base_behaviors
from functionaltests.api.v1.behaviors import container_behaviors
from functionaltests.api.v1.behaviors import secret_behaviors
from functionaltests.api.v1.models import order_models


class OrderBehaviors(base_behaviors.BaseBehaviors):

    def create_order(self, model, extra_headers=None, use_auth=True,
                     user_name=None, admin=None):
        """Create an order from the data in the model.

        :param model: The data used to create the order
        :param extra_headers: Optional HTTP headers to add to the request
        :param use_auth: Boolean to determine whether auth headers are sent
        :param user_name: the user used to do the create
        :param admin: the admin of the group to which user_name belongs
        :return: The create response and href for the order
        """

        # create the order
        resp = self.client.post('orders', request_model=model,
                                extra_headers=extra_headers,
                                user_name=user_name, use_auth=use_auth)

        # handle expected JSON parsing errors for unauthenticated requests
        if resp.status_code == 401 and not use_auth:
            return resp, None

        returned_data = self.get_json(resp)
        order_ref = returned_data.get('order_ref')

        # remember this order and its admin for our housekeeping cleanup
        if order_ref:
            if admin is None:
                admin = user_name
            self.created_entities.append((order_ref, admin))

        return resp, order_ref

    def get_order(self, order_ref, extra_headers=None, user_name=None,
                  use_auth=True):
        """Get an order from an href.

        :param order_ref: The href for an order
        :param extra_headers: Optional HTTP headers to add to the request
        :param user_name: the user used to do the get
        :param use_auth: Boolean to determine whether auth headers are sent
        :return: The response from the get
        """
        return self.client.get(order_ref,
                               response_model_type=order_models.OrderModel,
                               extra_headers=extra_headers,
                               user_name=user_name, use_auth=use_auth)

    def get_orders(self, limit=10, offset=0, filter=None,
                   extra_headers=None, user_name=None, use_auth=True):
        """Get a list of orders.

        :param limit: limits number of returned orders (default 10)
        :param offset: represents how many records to skip before retrieving
                       the list (default 0)
        :param filter: optional filter to limit the returned orders to
                        those whose metadata contains the filter.
        :param extra_headers: Optional HTTP headers to add to the request
        :param user_name: the user used to do the get
        :param use_auth: Boolean to determine whether auth headers are sent
        :return: the response, a list of orders and the next/pref hrefs
        """
        params = {'limit': limit, 'offset': offset}

        if filter:
            params['meta'] = filter

        resp = self.client.get('orders', params=params,
                               extra_headers=extra_headers,
                               user_name=user_name, use_auth=use_auth)

        # handle expected JSON parsing errors for unauthenticated requests
        if resp.status_code == 401 and not use_auth:
            return resp, None, None, None

        orders_list = self.get_json(resp)

        orders, next_ref, prev_ref = self.client.get_list_of_models(
            orders_list, order_models.OrderModel)

        return resp, orders, next_ref, prev_ref

    def delete_order(self, order_ref, extra_headers=None, expected_fail=False,
                     user_name=None, use_auth=True):
        """Delete an order.

        :param order_ref: HATEOAS ref of the order to be deleted
        :param extra_headers: Optional HTTP headers to add to the request
        :param expected_fail: Flag telling the delete whether or not this
                              operation is expected to fail (ie coming
                              from a negative test).  We need this to
                              determine whether or not this delete should
                              also remove an entity from our internal
                              list for housekeeping.
        :param user_name: the user used to do the delete
        :param use_auth: Boolean to determine whether auth headers are sent
        :return: A request response object
        """
        resp = self.client.delete(order_ref, extra_headers=extra_headers,
                                  user_name=user_name, use_auth=use_auth)

        if not expected_fail:
            for item in self.created_entities:
                if item[0] == order_ref:
                    self.created_entities.remove(item)

        return resp

    def delete_all_created_orders(self):
        """Delete all orders and other entities created by orders.

        """
        container_client = container_behaviors.ContainerBehaviors(self.client)
        secret_client = secret_behaviors.SecretBehaviors(self.client)

        orders_to_delete = [order for order in self.created_entities]

        for (order_ref, admin) in orders_to_delete:
            order_resp = self.get_order(order_ref, user_name=admin)

            # If order has secrets
            if order_resp.model.secret_ref:
                secret_client.delete_secret(order_resp.model.secret_ref,
                                            user_name=admin)

            # If containers supported
            container_attr_exists = getattr(order_resp.model,
                                            "container_ref",
                                            None)
            if container_attr_exists and order_resp.model.container_ref:
                container_resp = container_client.get_container(
                    order_resp.model.container_ref, user_name=admin)
                # remove secrets in the containers in the orders
                if container_resp.model.secret_refs:
                    for secret in container_resp.model.secret_refs:
                        secret_client.delete_secret(secret.secret_ref,
                                                    user_name=admin)

                container_client.delete_container(
                    order_resp.model.container_ref, user_name=admin)

            self.delete_order(order_ref, user_name=admin)



# Copyright (c) 2014 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import tempfile
from unittest import mock

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.primitives import serialization
import testtools

from barbican.tests import utils

try:
    import barbican.plugin.dogtag as dogtag_import
    import barbican.plugin.interface.secret_store as sstore

    import pki.key as dogtag_key
    imports_ok = True
except ImportError:
    # dogtag imports probably not available
    imports_ok = False


@testtools.skipIf(not imports_ok, "Dogtag imports not available")
class WhenTestingDogtagKRAPlugin(utils.BaseTestCase):

    def setUp(self):
        super(WhenTestingDogtagKRAPlugin, self).setUp()
        self.keyclient_mock = mock.MagicMock(name="KeyClient mock")
        self.patcher = mock.patch('pki.crypto.NSSCryptoProvider')
        self.patcher.start()

        # create nss db for test only
        self.nss_dir = tempfile.mkdtemp()

        self.plugin_name = "Test Dogtag KRA plugin"
        self.cfg_mock = mock.MagicMock(name='config mock')
        self.cfg_mock.dogtag_plugin = mock.MagicMock(
            nss_db_path=self.nss_dir,
            plugin_name=self.plugin_name,
            retries=3)
        self.plugin = dogtag_import.DogtagKRAPlugin(self.cfg_mock)
        self.plugin.keyclient = self.keyclient_mock

    def tearDown(self):
        super(WhenTestingDogtagKRAPlugin, self).tearDown()
        self.patcher.stop()
        os.rmdir(self.nss_dir)

    def test_get_plugin_name(self):
        self.assertEqual(self.plugin_name, self.plugin.get_plugin_name())

    def test_generate_symmetric_key(self):
        key_spec = sstore.KeySpec(sstore.KeyAlgorithm.AES, 128)
        self.plugin.generate_symmetric_key(key_spec)

        self.keyclient_mock.generate_symmetric_key.assert_called_once_with(
            mock.ANY,
            sstore.KeyAlgorithm.AES.upper(),
            128,
            mock.ANY)

    def test_generate_asymmetric_key(self):
        key_spec = sstore.KeySpec(sstore.KeyAlgorithm.RSA, 2048)
        self.plugin.generate_asymmetric_key(key_spec)

        self.keyclient_mock.generate_asymmetric_key.assert_called_once_with(
            mock.ANY,
            sstore.KeyAlgorithm.RSA.upper(),
            2048,
            mock.ANY)

    def test_generate_non_supported_algorithm(self):
        key_spec = sstore.KeySpec(sstore.KeyAlgorithm.EC, 192)
        self.assertRaises(
            dogtag_import.DogtagPluginAlgorithmException,
            self.plugin.generate_symmetric_key,
            key_spec
        )

    def test_raises_error_with_no_pem_path(self):
        m = mock.MagicMock()
        m.dogtag_plugin = mock.MagicMock(pem_path=None, nss_db_path='/tmp')
        self.assertRaises(
            ValueError,
            dogtag_import.DogtagKRAPlugin,
            m,
        )

    def test_store_secret(self):
        payload = 'encrypt me!!'
        key_spec = mock.MagicMock()
        content_type = mock.MagicMock()
        transport_key = None
        secret_dto = sstore.SecretDTO(sstore.SecretType.SYMMETRIC,
                                      payload,
                                      key_spec,
                                      content_type,
                                      transport_key)
        self.plugin.store_secret(secret_dto)
        self.keyclient_mock.archive_key.assert_called_once_with(
            mock.ANY,
            "passPhrase",
            payload,
            key_algorithm=None,
            key_size=None)

    def test_store_secret_with_tkey_id(self):
        payload = 'data wrapped in PKIArchiveOptions object'
        key_spec = mock.MagicMock()
        content_type = mock.MagicMock()
        transport_key = mock.MagicMock()
        secret_dto = sstore.SecretDTO(sstore.SecretType.SYMMETRIC,
                                      payload,
                                      key_spec,
                                      content_type,
                                      transport_key)
        self.plugin.store_secret(secret_dto)
        self.keyclient_mock.archive_pki_options.assert_called_once_with(
            mock.ANY,
            "passPhrase",
            payload,
            key_algorithm=None,
            key_size=None)

    def test_get_secret(self):
        secret_metadata = {
            dogtag_import.DogtagKRAPlugin.ALG: sstore.KeyAlgorithm.AES,
            dogtag_import.DogtagKRAPlugin.BIT_LENGTH: 256,
            dogtag_import.DogtagKRAPlugin.KEY_ID: 'key1'
        }
        self.plugin.get_secret(sstore.SecretType.SYMMETRIC, secret_metadata)

        self.keyclient_mock.retrieve_key.assert_called_once_with('key1', None)

    def test_get_secret_with_twsk(self):
        twsk = mock.MagicMock()
        secret_metadata = {
            dogtag_import.DogtagKRAPlugin.ALG: sstore.KeyAlgorithm.AES,
            dogtag_import.DogtagKRAPlugin.BIT_LENGTH: 256,
            dogtag_import.DogtagKRAPlugin.KEY_ID: 'key1',
            'trans_wrapped_session_key': twsk
        }
        self.plugin.get_secret(sstore.SecretType.SYMMETRIC, secret_metadata)

        self.keyclient_mock.retrieve_key.assert_called_once_with('key1', twsk)

    def test_get_private_key(self):
        test_key = rsa.generate_private_key(
            public_exponent=65537,
            key_size=2048,
            backend=default_backend()
        )
        key_data = dogtag_key.KeyData()
        key_data.data = test_key.private_bytes(
            serialization.Encoding.DER,
            serialization.PrivateFormat.PKCS8,
            serialization.NoEncryption())
        self.keyclient_mock.retrieve_key.return_value = key_data
        secret_metadata = {
            dogtag_import.DogtagKRAPlugin.ALG: sstore.KeyAlgorithm.RSA,
            dogtag_import.DogtagKRAPlugin.BIT_LENGTH: 2048,
            dogtag_import.DogtagKRAPlugin.KEY_ID: 'key1',
            dogtag_import.DogtagKRAPlugin.CONVERT_TO_PEM: 'true'
        }
        result = self.plugin.get_secret(sstore.SecretType.PRIVATE,
                                        secret_metadata)

        self.assertEqual(
            test_key.private_bytes(
                serialization.Encoding.PEM,
                serialization.PrivateFormat.PKCS8,
                serialization.NoEncryption()),
            result.secret
        )

    def test_get_public_key(self):
        test_public_key = rsa.generate_private_key(
            public_exponent=65537,
            key_size=2048,
            backend=default_backend()).public_key()
        key_info = dogtag_key.KeyInfo()
        key_info.public_key = test_public_key.public_bytes(
            serialization.Encoding.DER,
            serialization.PublicFormat.PKCS1)
        self.keyclient_mock.get_key_info.return_value = key_info
        secret_metadata = {
            dogtag_import.DogtagKRAPlugin.ALG: sstore.KeyAlgorithm.RSA,
            dogtag_import.DogtagKRAPlugin.BIT_LENGTH: 2048,
            dogtag_import.DogtagKRAPlugin.KEY_ID: 'key1',
            dogtag_import.DogtagKRAPlugin.CONVERT_TO_PEM: 'true'
        }
        result = self.plugin.get_secret(sstore.SecretType.PUBLIC,
                                        secret_metadata)

        self.assertEqual(
            test_public_key.public_bytes(
                serialization.Encoding.PEM,
                serialization.PublicFormat.PKCS1),
            result.secret
        )

    def test_store_passphrase_for_using_in_private_key_retrieval(self):

        key_spec = sstore.KeySpec(sstore.KeyAlgorithm.RSA, 2048,
                                  passphrase="password123")

        # Mock the response for passphrase archival
        request_response = dogtag_key.KeyRequestResponse()
        request_info = dogtag_key.KeyRequestInfo()
        request_info.key_url = "https://example_url/1"
        request_response.request_info = request_info
        self.keyclient_mock.archive_key.return_value = request_response

        asym_key_DTO = self.plugin.generate_asymmetric_key(key_spec)

        self.assertEqual(
            '1',
            asym_key_DTO.private_key_meta[
                dogtag_import.DogtagKRAPlugin.PASSPHRASE_KEY_ID]
        )

        self.keyclient_mock.generate_asymmetric_key.assert_called_once_with(
            mock.ANY,
            sstore.KeyAlgorithm.RSA.upper(),
            2048,
            mock.ANY)

    def test_supports_symmetric_aes_key_generation(self):
        key_spec = sstore.KeySpec(sstore.KeyAlgorithm.AES, 256)
        self.assertTrue(
            self.plugin.generate_supports(key_spec)
        )

    def test_supports_asymmetric_rsa_key_generation(self):
        key_spec = sstore.KeySpec(sstore.KeyAlgorithm.RSA, 2048)
        self.assertTrue(
            self.plugin.generate_supports(key_spec)
        )

    def test_supports_asymmetric_ec_key_generation(self):
        key_spec = sstore.KeySpec(sstore.KeyAlgorithm.EC, 156)
        self.assertFalse(
            self.plugin.generate_supports(key_spec)
        )

    def test_supports_symmetric_dh_key_generation(self):
        key_spec = sstore.KeySpec(sstore.KeyAlgorithm.DIFFIE_HELLMAN, 156)
        self.assertFalse(
            self.plugin.generate_supports(key_spec)
        )

    def test_does_not_support_unknown_type(self):
        key_spec = sstore.KeySpec("SOMETHING_RANDOM", 156)
        self.assertFalse(
            self.plugin.generate_supports(key_spec)
        )



#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.


import pecan

from barbican import api
from barbican.api import controllers
from barbican.common import hrefs
from barbican.common import utils
from barbican.common import validators
from barbican import i18n as u
from barbican.model import models
from barbican.model import repositories as repo

LOG = utils.getLogger(__name__)


def _convert_acl_to_response_format(acl, acls_dict):
    fields = acl.to_dict_fields()
    operation = fields['operation']

    acl_data = {}  # dict for each acl operation data

    acl_data['project-access'] = fields['project_access']
    acl_data['users'] = fields.get('users', [])
    acl_data['created'] = fields['created']
    acl_data['updated'] = fields['updated']

    acls_dict[operation] = acl_data


DEFAULT_ACL = {'read': {'project-access': True}}


class SecretACLsController(controllers.ACLMixin):
    """Handles SecretACL requests by a given secret id."""

    def __init__(self, secret):
        super().__init__()
        self.secret = secret
        self.secret_project_id = self.secret.project.external_id
        self.acl_repo = repo.get_secret_acl_repository()
        self.validator = validators.ACLValidator()

    def get_acl_tuple(self, req, **kwargs):
        d = {'project_id': self.secret_project_id,
             'creator_id': self.secret.creator_id}
        return 'secret', d

    @pecan.expose(generic=True)
    def index(self, **kwargs):
        pecan.abort(405)  # HTTP 405 Method Not Allowed as default

    @index.when(method='GET', template='json')
    @controllers.handle_exceptions(u._('SecretACL(s) retrieval'))
    @controllers.enforce_rbac('secret_acls:get')
    def on_get(self, external_project_id, **kw):
        LOG.debug('Start secret ACL on_get '
                  'for secret-ID %s:', self.secret.id)

        return self._return_acl_list_response(self.secret.id)

    @index.when(method='PATCH', template='json')
    @controllers.handle_exceptions(u._('SecretACL(s) Update'))
    @controllers.enforce_rbac('secret_acls:put_patch')
    @controllers.enforce_content_types(['application/json'])
    def on_patch(self, external_project_id, **kwargs):
        """Handles update of existing secret acl requests.

        At least one secret ACL needs to exist for update to proceed.
        In update, multiple operation ACL payload can be specified as
        mentioned in sample below. A specific ACL can be updated by its
        own id via SecretACLController patch request.

        {
          "read":{
            "users":[
              "5ecb18f341894e94baca9e8c7b6a824a",
              "20b63d71f90848cf827ee48074f213b7",
              "c7753f8da8dc4fbea75730ab0b6e0ef4"
            ]
          },
          "write":{
            "users":[
              "5ecb18f341894e94baca9e8c7b6a824a"
            ],
            "project-access":true
          }
        }
        """
        data = api.load_body(pecan.request, validator=self.validator)
        LOG.debug('Start on_patch...%s', data)

        existing_acls_map = {acl.operation: acl for acl in
                             self.secret.secret_acls}
        for operation in filter(lambda x: data.get(x),
                                validators.ACL_OPERATIONS):
            project_access = data[operation].get('project-access')
            user_ids = data[operation].get('users')
            s_acl = None
            if operation in existing_acls_map:  # update if matching acl exists
                s_acl = existing_acls_map[operation]
                if project_access is not None:
                    s_acl.project_access = project_access
            else:
                s_acl = models.SecretACL(self.secret.id, operation=operation,
                                         project_access=project_access)
            self.acl_repo.create_or_replace_from(self.secret, secret_acl=s_acl,
                                                 user_ids=user_ids)

        acl_ref = '{0}/acl'.format(
            hrefs.convert_secret_to_href(self.secret.id))
        return {'acl_ref': acl_ref}

    @index.when(method='PUT', template='json')
    @controllers.handle_exceptions(u._('SecretACL(s) Update'))
    @controllers.enforce_rbac('secret_acls:put_patch')
    @controllers.enforce_content_types(['application/json'])
    def on_put(self, external_project_id, **kwargs):
        """Handles update of existing secret acl requests.

        Replaces existing secret ACL(s) with input ACL(s) data. Existing
        ACL operation not specified in input are removed as part of update.
        For missing project-access in ACL, true is used as default.
        In update, multiple operation ACL payload can be specified as
        mentioned in sample below. A specific ACL can be updated by its
        own id via SecretACLController patch request.

        {
          "read":{
            "users":[
              "5ecb18f341894e94baca9e8c7b6a824a",
              "20b63d71f90848cf827ee48074f213b7",
              "c7753f8da8dc4fbea75730ab0b6e0ef4"
            ]
          },
          "write":{
            "users":[
              "5ecb18f341894e94baca9e8c7b6a824a"
            ],
            "project-access":false
          }
        }

        Every secret, by default, has an implicit ACL in case client has not
        defined an explicit ACL. That default ACL definition, DEFAULT_ACL,
        signifies that a secret by default has project based access i.e. client
        with necessary roles on secret project can access the secret. That's
        why when ACL is added to a secret, it always returns 200 (and not 201)
        indicating existence of implicit ACL on a secret.
        """
        data = api.load_body(pecan.request, validator=self.validator)
        LOG.debug('Start on_put...%s', data)

        existing_acls_map = {acl.operation: acl for acl in
                             self.secret.secret_acls}
        for operation in filter(lambda x: data.get(x),
                                validators.ACL_OPERATIONS):
            project_access = data[operation].get('project-access', True)
            user_ids = data[operation].get('users', [])
            s_acl = None
            if operation in existing_acls_map:  # update if matching acl exists
                s_acl = existing_acls_map.pop(operation)
                s_acl.project_access = project_access
            else:
                s_acl = models.SecretACL(self.secret.id, operation=operation,
                                         project_access=project_access)
            self.acl_repo.create_or_replace_from(self.secret, secret_acl=s_acl,
                                                 user_ids=user_ids)
        # delete remaining existing acls as they are not present in input.
        for acl in existing_acls_map.values():
            self.acl_repo.delete_entity_by_id(entity_id=acl.id,
                                              external_project_id=None)
        acl_ref = '{0}/acl'.format(
            hrefs.convert_secret_to_href(self.secret.id))
        return {'acl_ref': acl_ref}

    @index.when(method='DELETE', template='json')
    @controllers.handle_exceptions(u._('SecretACL(s) deletion'))
    @controllers.enforce_rbac('secret_acls:delete')
    def on_delete(self, external_project_id, **kwargs):

        count = self.acl_repo.get_count(self.secret.id)
        if count > 0:
            self.acl_repo.delete_acls_for_secret(self.secret)

    def _return_acl_list_response(self, secret_id):
        result = self.acl_repo.get_by_secret_id(secret_id)

        acls_data = {}
        if result:
            for acl in result:
                _convert_acl_to_response_format(acl, acls_data)
        if not acls_data:
            acls_data = DEFAULT_ACL.copy()
        return acls_data


class ContainerACLsController(controllers.ACLMixin):
    """Handles ContainerACL requests by a given container id."""

    def __init__(self, container):
        super().__init__()
        self.container = container
        self.container_id = container.id
        self.acl_repo = repo.get_container_acl_repository()
        self.container_repo = repo.get_container_repository()
        self.validator = validators.ACLValidator()
        self.container_project_id = container.project.external_id

    def get_acl_tuple(self, req, **kwargs):
        d = {'project_id': self.container_project_id,
             'creator_id': self.container.creator_id}
        return 'container', d

    @pecan.expose(generic=True)
    def index(self, **kwargs):
        pecan.abort(405)  # HTTP 405 Method Not Allowed as default

    @index.when(method='GET', template='json')
    @controllers.handle_exceptions(u._('ContainerACL(s) retrieval'))
    @controllers.enforce_rbac('container_acls:get')
    def on_get(self, external_project_id, **kw):
        LOG.debug('Start container ACL on_get '
                  'for container-ID %s:', self.container_id)

        return self._return_acl_list_response(self.container.id)

    @index.when(method='PATCH', template='json')
    @controllers.handle_exceptions(u._('ContainerACL(s) Update'))
    @controllers.enforce_rbac('container_acls:put_patch')
    @controllers.enforce_content_types(['application/json'])
    def on_patch(self, external_project_id, **kwargs):
        """Handles update of existing container acl requests.

        At least one container ACL needs to exist for update to proceed.
        In update, multiple operation ACL payload can be specified as
        mentioned in sample below. A specific ACL can be updated by its
        own id via ContainerACLController patch request.

        {
          "read":{
            "users":[
              "5ecb18f341894e94baca9e8c7b6a824a",
              "20b63d71f90848cf827ee48074f213b7",
              "c7753f8da8dc4fbea75730ab0b6e0ef4"
            ]
          },
          "write":{
            "users":[
              "5ecb18f341894e94baca9e8c7b6a824a"
            ],
            "project-access":false
          }
        }
        """
        data = api.load_body(pecan.request, validator=self.validator)
        LOG.debug('Start ContainerACLsController on_patch...%s', data)

        existing_acls_map = {acl.operation: acl for acl in
                             self.container.container_acls}
        for operation in filter(lambda x: data.get(x),
                                validators.ACL_OPERATIONS):
            project_access = data[operation].get('project-access')
            user_ids = data[operation].get('users')
            if operation in existing_acls_map:  # update if matching acl exists
                c_acl = existing_acls_map[operation]
                if project_access is not None:
                    c_acl.project_access = project_access
            else:
                c_acl = models.ContainerACL(self.container.id,
                                            operation=operation,
                                            project_access=project_access)
            self.acl_repo.create_or_replace_from(self.container,
                                                 container_acl=c_acl,
                                                 user_ids=user_ids)

        acl_ref = '{0}/acl'.format(
            hrefs.convert_container_to_href(self.container.id))
        return {'acl_ref': acl_ref}

    @index.when(method='PUT', template='json')
    @controllers.handle_exceptions(u._('ContainerACL(s) Update'))
    @controllers.enforce_rbac('container_acls:put_patch')
    @controllers.enforce_content_types(['application/json'])
    def on_put(self, external_project_id, **kwargs):
        """Handles update of existing container acl requests.

        Replaces existing container ACL(s) with input ACL(s) data. Existing
        ACL operation not specified in input are removed as part of update.
        For missing project-access in ACL, true is used as default.
        In update, multiple operation ACL payload can be specified as
        mentioned in sample below. A specific ACL can be updated by its
        own id via ContainerACLController patch request.

        {
          "read":{
            "users":[
              "5ecb18f341894e94baca9e8c7b6a824a",
              "20b63d71f90848cf827ee48074f213b7",
              "c7753f8da8dc4fbea75730ab0b6e0ef4"
            ]
          },
          "write":{
            "users":[
              "5ecb18f341894e94baca9e8c7b6a824a"
            ],
            "project-access":false
          }
        }

        Every container, by default, has an implicit ACL in case client has not
        defined an explicit ACL. That default ACL definition, DEFAULT_ACL,
        signifies that a container by default has project based access i.e.
        client with necessary roles on container project can access the
        container. That's why when ACL is added to a container, it always
        returns 200 (and not 201) indicating existence of implicit ACL on a
        container.
        """

        data = api.load_body(pecan.request, validator=self.validator)
        LOG.debug('Start ContainerACLsController on_put...%s', data)

        existing_acls_map = {acl.operation: acl for acl in
                             self.container.container_acls}
        for operation in filter(lambda x: data.get(x),
                                validators.ACL_OPERATIONS):
            project_access = data[operation].get('project-access', True)
            user_ids = data[operation].get('users', [])
            if operation in existing_acls_map:  # update if matching acl exists
                c_acl = existing_acls_map.pop(operation)
                c_acl.project_access = project_access
            else:
                c_acl = models.ContainerACL(self.container.id,
                                            operation=operation,
                                            project_access=project_access)
            self.acl_repo.create_or_replace_from(self.container,
                                                 container_acl=c_acl,
                                                 user_ids=user_ids)
        # delete remaining existing acls as they are not present in input.
        for acl in existing_acls_map.values():
            self.acl_repo.delete_entity_by_id(entity_id=acl.id,
                                              external_project_id=None)
        acl_ref = '{0}/acl'.format(
            hrefs.convert_container_to_href(self.container.id))
        return {'acl_ref': acl_ref}

    @index.when(method='DELETE', template='json')
    @controllers.handle_exceptions(u._('ContainerACL(s) deletion'))
    @controllers.enforce_rbac('container_acls:delete')
    def on_delete(self, external_project_id, **kwargs):
        count = self.acl_repo.get_count(self.container_id)
        if count > 0:
            self.acl_repo.delete_acls_for_container(self.container)

    def _return_acl_list_response(self, container_id):
        result = self.acl_repo.get_by_container_id(container_id)

        acls_data = {}
        if result:
            for acl in result:
                _convert_acl_to_response_format(acl, acls_data)
        if not acls_data:
            acls_data = DEFAULT_ACL.copy()
        return acls_data



#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from oslo_log import versionutils
from oslo_policy import policy

from barbican.common.policies import base


deprecated_orders_get = policy.DeprecatedRule(
    name='orders:get',
    check_str='rule:all_but_audit',
    deprecated_reason=base.LEGACY_POLICY_DEPRECATION,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_orders_post = policy.DeprecatedRule(
    name='orders:post',
    check_str='rule:admin_or_creator',
    deprecated_reason=base.LEGACY_POLICY_DEPRECATION,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_orders_put = policy.DeprecatedRule(
    name='orders:put',
    check_str='rule:admin_or_creator',
    deprecated_reason=base.LEGACY_POLICY_DEPRECATION,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_order_get = policy.DeprecatedRule(
    name='order:get',
    check_str='rule:all_users and project_id:%(target.order.project_id)s',
    deprecated_reason=base.LEGACY_POLICY_DEPRECATION,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_order_delete = policy.DeprecatedRule(
    name='order:delete',
    check_str='rule:admin and project_id:%(target.order.project_id)s',
    deprecated_reason=base.LEGACY_POLICY_DEPRECATION,
    deprecated_since=versionutils.deprecated.WALLABY
)

rules = [
    policy.DocumentedRuleDefault(
        name='orders:get',
        check_str="True:%(enforce_new_defaults)s and role:member",
        scope_types=['project'],
        description='Gets list of all orders associated with a project.',
        operations=[
            {
                'path': '/v1/orders',
                'method': 'GET'
            }
        ],
        deprecated_rule=deprecated_orders_get
    ),
    policy.DocumentedRuleDefault(
        name='orders:post',
        check_str="True:%(enforce_new_defaults)s and role:member",
        scope_types=['project'],
        description='Creates an order.',
        operations=[
            {
                'path': '/v1/orders',
                'method': 'POST'
            }
        ],
        deprecated_rule=deprecated_orders_post

    ),
    policy.DocumentedRuleDefault(
        name='orders:put',
        check_str="True:%(enforce_new_defaults)s and role:member",
        scope_types=['project'],
        description='Unsupported method for the orders API.',
        operations=[
            {
                'path': '/v1/orders',
                'method': 'PUT'
            }
        ],
        deprecated_rule=deprecated_orders_put
    ),
    policy.DocumentedRuleDefault(
        name='order:get',
        check_str=(
            "True:%(enforce_new_defaults)s and "
            "rule:order_project_member"),
        scope_types=['project'],
        description='Retrieves an orders metadata.',
        operations=[
            {
                'path': '/v1/orders/{order-id}',
                'method': 'GET'
            }
        ],
        deprecated_rule=deprecated_order_get
    ),
    policy.DocumentedRuleDefault(
        name='order:delete',
        check_str=(
            "True:%(enforce_new_defaults)s and "
            "rule:order_project_member"),
        scope_types=['project'],
        description='Deletes an order.',
        operations=[
            {
                'path': '/v1/orders/{order-id}',
                'method': 'DELETE'
            }
        ],
        deprecated_rule=deprecated_order_delete
    )
]


def list_rules():
    return rules



# Copyright 2025 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import abc
import traceback

from oslo_db.sqlalchemy import session
from sqlalchemy import orm
from sqlalchemy.orm import scoping

from barbican.model import models


class BaseKEKRewrap(metaclass=abc.ABCMeta):

    def __init__(self, conf):
        self.db_engine = session.create_engine(conf.database.connection)
        self._session_creator = scoping.scoped_session(
            orm.sessionmaker(
                bind=self.db_engine,
            )
        )

    @abc.abstractmethod
    def rewrap_kek(self, project, kek):
        raise NotImplementedError

    @property
    def db_session(self):
        return self._session_creator()

    def get_keks_for_project(self, project):
        keks = []
        with self.db_session.begin() as transaction:
            print('Retrieving KEKs for Project {}'.format(project.id))
            query = transaction.session.query(models.KEKDatum)
            query = query.filter_by(project_id=project.id)
            query = query.filter_by(plugin_name=self.plugin_name)

            keks = query.all()

        return keks

    def get_projects(self):
        print('Retrieving all available projects')

        projects = []
        with self.db_session.begin() as transaction:
            projects = transaction.session.query(models.Project).all()

        return projects

    def execute(self, dry_run=True):
        self.dry_run = dry_run
        if self.dry_run:
            print('-- Running in dry-run mode --')

        projects = self.get_projects()
        for project in projects:
            keks = self.get_keks_for_project(project)
            for kek in keks:
                try:
                    self.rewrap_kek(project, kek)
                except Exception:
                    print('Error occurred! SQLAlchemy automatically rolled-'
                          'back the transaction')
                    traceback.print_exc()



#    Copyright 2018 Fujitsu.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from oslo_versionedobjects import base as object_base

from barbican.model import models
from barbican.model import repositories as repo
from barbican.objects import base
from barbican.objects import fields


@object_base.VersionedObjectRegistry.register
class KEKDatum(base.BarbicanObject, base.BarbicanPersistentObject,
               object_base.VersionedObjectDictCompat):

    fields = {
        'plugin_name': fields.StringField(nullable=True, default=None),
        'kek_label': fields.StringField(nullable=True, default=None),
        'project_id': fields.StringField(nullable=True, default=None),
        'active': fields.BooleanField(default=True),
        'bind_completed': fields.BooleanField(default=False),
        'algorithm': fields.StringField(nullable=True, default=None),
        'bit_length': fields.IntegerField(nullable=True, default=None),
        'mode': fields.StringField(nullable=True, default=None),
        'plugin_meta': fields.StringField(nullable=True, default=None)
    }

    db_model = models.KEKDatum
    db_repo = repo.get_kek_datum_repository()

    @classmethod
    def find_or_create_kek_datum(cls, project, plugin_name,
                                 suppress_exception=False, session=None):
        kek_datum_db = cls.db_repo.find_or_create_kek_datum(
            project, plugin_name, suppress_exception, session)
        return cls()._from_db_object(kek_datum_db)



# Copyright (c) 2014 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import copy

from testtools import testcase

from barbican.tests import utils
from functionaltests.api import base
from functionaltests.api.v1.behaviors import container_behaviors
from functionaltests.api.v1.behaviors import secret_behaviors
from functionaltests.api.v1.models import container_models
from functionaltests.api.v1.models import secret_models


def get_default_container_create_data(secret):
    return {
        "type": "generic",
        "name": "generic name",
        "secret_refs": [
            {
                "name": "a secret",
                "secret_ref": secret
            }
        ]
    }


create_container_data = {
    "name": "containername",
    "type": "generic",
    "secret_refs": [
        {
            "name": "secret1",
        },
        {
            "name": "secret2",
        },
        {
            "name": "secret3"
        }
    ]
}

create_container_rsa_data = {
    "name": "rsacontainer",
    "type": "rsa",
    "secret_refs": [
        {
            "name": "public_key",
        },
        {
            "name": "private_key",
        },
        {
            "name": "private_key_passphrase"
        }
    ]
}

accepted_str_values = {
    'alphanumeric': ['a2j3j6ll9'],
    'punctuation': ['~!@#$%^&*()_+`-={}[]|:;<>,.?'],
    'len_255': [base.TestCase.max_sized_field],
    'uuid': ['54262d9d-4bc7-4821-8df0-dc2ca8e112bb'],
    'empty': ['']
}


class BaseContainerTestCase(base.TestCase):
    default_data_template = create_container_data

    def setUp(self):
        super(BaseContainerTestCase, self).setUp()
        self.secret_behaviors = secret_behaviors.SecretBehaviors(self.client)
        self.behaviors = container_behaviors.ContainerBehaviors(self.client)

        # Setting up three secrets for building containers
        self.secret_ref_1 = self._create_a_secret()
        self.secret_ref_2 = self._create_a_secret()
        self.secret_ref_3 = self._create_a_secret()

        self.default_data = copy.deepcopy(self.default_data_template)

        default_secret_refs = self.default_data['secret_refs']
        default_secret_refs[0]['secret_ref'] = self.secret_ref_1
        default_secret_refs[1]['secret_ref'] = self.secret_ref_2
        default_secret_refs[2]['secret_ref'] = self.secret_ref_3

    def tearDown(self):
        self.secret_behaviors.delete_all_created_secrets()
        self.behaviors.delete_all_created_containers()
        super(BaseContainerTestCase, self).tearDown()

    def _cleanup_all_containers(self):
        for user_name in self.client.get_all_functional_test_user_names():
            self.behaviors.delete_all_containers_for_user(user_name=user_name)

    def _create_a_secret(self):
        secret_defaults_data = {
            "name": "AES key",
            "expiration": "2030-02-28T19:14:44.180394",
            "algorithm": "aes",
            "bit_length": 256,
            "mode": "cbc",
            "payload": "gF6+lLoF3ohA9aPRpt+6bQ==",
            "payload_content_type": "application/octet-stream",
            "payload_content_encoding": "base64",
        }

        secret_model = secret_models.SecretModel(**secret_defaults_data)
        resp, secret_ref = self.secret_behaviors.create_secret(secret_model)
        self.assertEqual(201, resp.status_code)
        self.assertIsNotNone(secret_ref)

        return secret_ref


@utils.parameterized_test_case
class GenericContainersTestCase(BaseContainerTestCase):

    @testcase.attr('positive')
    def test_create_defaults_none_secret_name(self):
        """Covers creating a container with None as a secret name."""
        test_model = container_models.ContainerModel(**self.default_data)
        test_model.name = None

        resp, container_ref = self.behaviors.create_container(test_model)
        self.assertEqual(201, resp.status_code)

    @utils.parameterized_dataset({'0': [0], '1': [1], '50': [50]})
    @testcase.attr('positive')
    def test_create_defaults_size(self, num_secrets):
        """Covers creating containers of various sizes."""
        test_model = container_models.ContainerModel(**self.default_data)
        for i in range(0, num_secrets):
            secret_ref = self._create_a_secret()
            test_model.secret_refs.append({
                'name': 'other_secret{0}'.format(i),
                'secret_ref': secret_ref
            })

        resp, container_ref = self.behaviors.create_container(test_model)
        self.assertEqual(201, resp.status_code)

    @utils.parameterized_dataset(accepted_str_values)
    @testcase.attr('positive')
    def test_create_defaults_name(self, name):
        """Covers creating generic containers with various names."""
        test_model = container_models.ContainerModel(**self.default_data)
        test_model.name = name

        resp, container_ref = self.behaviors.create_container(test_model)
        self.assertEqual(201, resp.status_code)

    @utils.parameterized_dataset(accepted_str_values)
    @testcase.attr('positive')
    def test_create_defaults_secret_name(self, name=None):
        """Covers creating containers with various secret ref names."""
        test_model = container_models.ContainerModel(**self.default_data)
        test_model.secret_refs = [{
            'name': name,
            'secret_ref': self.secret_ref_1
        }]

        resp, container_ref = self.behaviors.create_container(test_model)
        self.assertEqual(201, resp.status_code)

        get_resp = self.behaviors.get_container(container_ref)
        self.assertEqual(200, get_resp.status_code)
        self.assertEqual(get_resp.model.secret_refs[0].name, name)

    @testcase.attr('negative')
    def test_create_defaults_invalid_type(self):
        """Container creating should fail with an invalid container type."""
        test_model = container_models.ContainerModel(**self.default_data)
        test_model.type = 'bad_type'

        resp, container_ref = self.behaviors.create_container(test_model)
        self.assertEqual(400, resp.status_code)

    @testcase.attr('negative')
    def test_create_defaults_duplicate_secret_refs(self):
        """Covers creating a container with a duplicated secret ref."""
        test_model = container_models.ContainerModel(**self.default_data)
        test_model.secret_refs[0]['secret_ref'] = self.secret_ref_1
        test_model.secret_refs[1]['secret_ref'] = self.secret_ref_1
        test_model.secret_refs[2]['secret_ref'] = self.secret_ref_1

        resp, container_ref = self.behaviors.create_container(test_model)
        self.assertEqual(400, resp.status_code)

    @testcase.attr('negative')
    def test_get_non_existent_container(self):
        """A get on a container that does not exist should return a 404."""
        resp = self.behaviors.get_container("not_a_ref")
        self.assertEqual(404, resp.status_code)

    @testcase.attr('negative')
    def test_delete_non_existent_container(self):
        """A delete on a container that does not exist should return a 404."""
        resp = self.behaviors.delete_container("not_a_ref", expected_fail=True)
        self.assertEqual(404, resp.status_code)

    @testcase.skipIf(not base.conf_host_href_used, 'response href using '
                     'wsgi request instead of CONF.host_href')
    @testcase.attr('positive')
    def test_create_change_host_with_header_not_allowed(self, **kwargs):
        """Create a container with a (possibly) malicious host name header."""

        test_model = container_models.ContainerModel(**self.default_data)

        malicious_hostname = 'some.bad.server.com'
        changed_host_header = {'Host': malicious_hostname}

        resp, container_ref = self.behaviors.create_container(
            test_model, extra_headers=changed_host_header)

        self.assertEqual(201, resp.status_code)

        # get Location field from result and assert that it is NOT the
        # malicious one.
        regex = '.*{0}.*'.format(malicious_hostname)
        self.assertNotRegex(resp.headers['location'], regex)

    @testcase.skipIf(base.conf_host_href_used, 'response href using '
                     'CONF.host_href instead of wsgi request')
    @testcase.attr('positive')
    def test_get_change_host_with_header_allowed(self, **kwargs):
        """Get a container with a alternative proxy host name header."""

        test_model = container_models.ContainerModel(**self.default_data)

        another_proxy_hostname = 'proxy2.server.com'
        changed_host_header = {'Host': another_proxy_hostname}

        # In test, cannot pass different host header during create as returned
        # container_href in response contains that host in url. That url is
        # used in deleting that container during cleanup step.
        resp, container_href = self.behaviors.create_container(test_model)
        self.assertEqual(201, resp.status_code)

        resp = self.behaviors.get_container(container_href,
                                            extra_headers=changed_host_header)
        # Assert that returned href has provided proxy hostname
        regex = '.*{0}.*'.format(another_proxy_hostname)
        self.assertRegex(resp.model.container_ref, regex)


@utils.parameterized_test_case
class RSAContainersTestCase(BaseContainerTestCase):
    default_data_template = create_container_rsa_data

    @testcase.attr('positive')
    def test_create_rsa_no_passphrase(self):
        """Covers creating an rsa container without a passphrase."""
        pub_key_ref = {'name': 'public_key', 'secret_ref': self.secret_ref_1}
        priv_key_ref = {'name': 'private_key', 'secret_ref': self.secret_ref_2}

        test_model = container_models.ContainerModel(**self.default_data)
        test_model.secret_refs = [pub_key_ref, priv_key_ref]

        resp, container_ref = self.behaviors.create_container(test_model)
        self.assertEqual(201, resp.status_code)

    @utils.parameterized_dataset(accepted_str_values)
    @testcase.attr('positive')
    def test_create_rsa_name(self, name):
        """Covers creating rsa containers with various names."""
        test_model = container_models.ContainerModel(**self.default_data)
        test_model.name = name

        resp, container_ref = self.behaviors.create_container(test_model)
        self.assertEqual(201, resp.status_code)

    @testcase.attr('negative')
    def test_create_rsa_invalid_key_names(self):
        """Covers creating an RSA container with incorrect names."""
        test_model = container_models.ContainerModel(**self.default_data)
        test_model.secret_refs = [
            {
                "name": "secret1",
                "secret_ref": self.secret_ref_1
            },
            {
                "name": "secret2",
                "secret_ref": self.secret_ref_2
            },
            {
                "name": "secret3",
                "secret_ref": self.secret_ref_3
            }
        ]

        resp, container_ref = self.behaviors.create_container(test_model)
        self.assertEqual(400, resp.status_code)

    @testcase.attr('negative')
    def test_create_rsa_no_public_key(self):
        """Creating an rsa container without a public key should fail.

        RSA containers must have at least a public key and private key.
        """
        test_model = container_models.ContainerModel(**self.default_data)
        test_model.secret_refs[0]['name'] = 'secret_1'

        resp, container_ref = self.behaviors.create_container(test_model)
        self.assertEqual(400, resp.status_code)

    @testcase.attr('negative')
    def test_create_rsa_no_private_key(self):
        """Creating an rsa container without a private key should fail.

        RSA containers must have at least a public key and private key.
        """
        test_model = container_models.ContainerModel(**self.default_data)
        test_model.secret_refs[1]['name'] = 'secret_1'

        resp, container_ref = self.behaviors.create_container(test_model)
        self.assertEqual(400, resp.status_code)

    @testcase.skipIf(not base.conf_host_href_used, 'response href using '
                     'wsgi request instead of CONF.host_href')
    @testcase.attr('positive')
    def test_create_rsa_change_host_with_header_not_allowed(self, **kwargs):
        """Create a container with a (possibly) malicious host name header."""

        test_model = container_models.ContainerModel(**self.default_data)

        malicious_hostname = 'some.bad.server.com'
        changed_host_header = {'Host': malicious_hostname}

        resp, container_ref = self.behaviors.create_container(
            test_model, extra_headers=changed_host_header)

        self.assertEqual(201, resp.status_code)

        # get Location field from result and assert that it is NOT the
        # malicious one.
        regex = '.*{0}.*'.format(malicious_hostname)
        self.assertNotRegex(resp.headers['location'], regex)

    @testcase.skipIf(base.conf_host_href_used, 'response href using '
                     'CONF.host_href instead of wsgi request')
    @testcase.attr('positive')
    def test_get_rsa_change_host_with_header_allowed(self, **kwargs):
        """Get a container with a alternative proxy host name header."""

        test_model = container_models.ContainerModel(**self.default_data)

        another_proxy_hostname = 'proxy2.server.com'
        changed_host_header = {'Host': another_proxy_hostname}

        # In test, cannot pass different host header during create as returned
        # container_href in response contains that host in url. That url is
        # used in deleting that container during cleanup step.
        resp, container_href = self.behaviors.create_container(test_model)
        self.assertEqual(201, resp.status_code)

        resp = self.behaviors.get_container(container_href,
                                            extra_headers=changed_host_header)
        # Assert that returned href has provided proxy hostname
        regex = '.*{0}.*'.format(another_proxy_hostname)
        self.assertRegex(resp.model.container_ref, regex)


class ContainersPagingTestCase(base.PagingTestCase):

    def setUp(self):
        super(ContainersPagingTestCase, self).setUp()
        self.secret_behaviors = secret_behaviors.SecretBehaviors(self.client)
        self.behaviors = container_behaviors.ContainerBehaviors(self.client)

        # make a local mutable copy of the default data to prevent
        # possible data contamination
        secret = self._create_a_secret()
        self.create_default_data = get_default_container_create_data(secret)

    def _create_a_secret(self):
        secret_defaults_data = {
            "name": "AES key",
            "expiration": "2030-02-28T19:14:44.180394",
            "algorithm": "aes",
            "bit_length": 256,
            "mode": "cbc",
            "payload": "gF6+lLoF3ohA9aPRpt+6bQ==",
            "payload_content_type": "application/octet-stream",
            "payload_content_encoding": "base64",
        }

        secret_model = secret_models.SecretModel(**secret_defaults_data)
        resp, secret_ref = self.secret_behaviors.create_secret(secret_model)
        self.assertEqual(201, resp.status_code)
        self.assertIsNotNone(secret_ref)

        return secret_ref

    def tearDown(self):
        self.secret_behaviors.delete_all_created_secrets()
        self.behaviors.delete_all_created_containers()
        super(ContainersPagingTestCase, self).tearDown()

    def create_model(self):
        return container_models.ContainerModel(**self.create_default_data)

    def create_resources(self, count=0, model=None):
        for x in range(0, count):
            self.behaviors.create_container(model)

    def get_resources(self, limit=10, offset=0, filter=filter):
        return self.behaviors.get_containers(limit=limit, offset=offset,
                                             filter=filter)

    def set_filter_field(self, unique_str, model):
        '''Set the name field which we use in the get_resources '''
        model.name = unique_str


class ContainersUnauthedTestCase(BaseContainerTestCase):

    def setUp(self):
        super(ContainersUnauthedTestCase, self).setUp()

        self.dummy_project_id = 'dummy123'
        self.dummy_container_ref = (
            'containers/dummy123-3416-4b53-8875-e6af3e0af8c3'
        )

    def tearDown(self):
        self.secret_behaviors.delete_all_created_secrets()
        super(ContainersUnauthedTestCase, self).tearDown()

    @testcase.attr('negative', 'security')
    def test_unauthed_create_huge_dummy_token_no_proj_id(self):
        """Attempt to create a container with a dummy token, and no project id

        Should return 401
        """

        model = container_models.ContainerModel(
            **create_container_data
        )
        headers = {'X-Auth-Token': 'a' * 3500}
        resp = self.client.post(
            'containers', request_model=model, use_auth=False,
            extra_headers=headers
        )
        self.assertEqual(401, resp.status_code)

    @testcase.attr('negative', 'security')
    def test_unauthed_create_no_proj_id(self):
        """Attempt to create a container without a token or project id

        Should return 401
        """

        model = container_models.ContainerModel(
            **create_container_data
        )
        resp = self.client.post(
            'containers', request_model=model, use_auth=False
        )
        self.assertEqual(401, resp.status_code)

    @testcase.attr('negative', 'security')
    def test_unauthed_get_no_proj_id(self):
        """Attempt to get a container without a token or project id

        Should return 401
        """

        headers = {
            'Accept': '*/*',
            'Accept-Encoding': '*/*'
        }
        resp = self.client.get(
            self.dummy_container_ref, extra_headers=headers, use_auth=False
        )
        self.assertEqual(401, resp.status_code)

    @testcase.attr('negative', 'security')
    def test_unauthed_delete_no_proj_id(self):
        """Attempt to delete a container without a token or project id

        Should return 401
        """

        resp = self.client.delete(self.dummy_container_ref, use_auth=False)
        self.assertEqual(401, resp.status_code)

    @testcase.attr('negative', 'security')
    def test_unauthed_huge_dummy_token_with_proj_id(self):
        """Attempt to create a container with a dummy token and project id

        Should return 401
        """

        model = container_models.ContainerModel(
            **create_container_data
        )
        headers = {
            'X-Auth-Token': 'a' * 3500,
            'X-Project-Id': self.dummy_project_id
        }
        resp = self.client.post(
            'containers', request_model=model, use_auth=False,
            extra_headers=headers
        )
        self.assertEqual(401, resp.status_code)

    @testcase.attr('negative', 'security')
    def test_unauthed_create_with_proj_id(self):
        """Attempt to create a container with a project id, but not a token

        Should return 401
        """

        model = container_models.ContainerModel(
            **create_container_data
        )
        headers = {'X-Project-Id': self.dummy_project_id}
        resp = self.client.post(
            'containers', request_model=model, use_auth=False,
            extra_headers=headers
        )
        self.assertEqual(401, resp.status_code)

    @testcase.attr('negative', 'security')
    def test_unauthed_get_with_proj_id(self):
        """Attempt to get a container with a project id, but not a token

        Should return 401
        """

        headers = {
            'Accept': '*/*',
            'Accept-Encoding': '*/*',
            'X-Project-Id': self.dummy_project_id
        }
        resp = self.client.get(
            self.dummy_container_ref, extra_headers=headers, use_auth=False
        )
        self.assertEqual(401, resp.status_code)

    @testcase.attr('negative', 'security')
    def test_unauthed_delete_with_proj_id(self):
        """Attempt to delete a container with a project id, but not a token

        Should return 401
        """

        headers = {'X-Project-Id': self.dummy_project_id}
        resp = self.client.delete(
            self.dummy_container_ref, use_auth=False, extra_headers=headers
        )
        self.assertEqual(401, resp.status_code)



#    Copyright 2018 Fujitsu.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""Barbican common internal objects model"""

from oslo_versionedobjects import base as object_base

from barbican.model import repositories as repos
from barbican.objects import fields


class States(object):
    PENDING = 'PENDING'
    ACTIVE = 'ACTIVE'
    ERROR = 'ERROR'

    @classmethod
    def is_valid(cls, state_to_test):
        """Tests if a state is a valid one."""
        return state_to_test in cls.__dict__


class BarbicanPersistentObject(object):
    fields = {
        'id': fields.StringField(nullable=True, default=None),
        'created_at': fields.DateTimeField(nullable=True, tzinfo_aware=False),
        'updated_at': fields.DateTimeField(nullable=True, tzinfo_aware=False),
        'deleted_at': fields.DateTimeField(nullable=True, tzinfo_aware=False),
        'deleted': fields.BooleanField(nullable=True),
        'status': fields.StringField(nullable=True, default=States.PENDING)
    }


class BarbicanObject(object_base.VersionedObject):

    # Version 1.0: Initial version
    VERSION = '1.0'

    OBJ_PROJECT_NAMESPACE = 'barbican'
    synthetic_fields = []

    # NOTE(namnh): The db_model, db_repo class variable should be inherited
    # by sub-classes.
    # For example, in the Secret object has to have "db_model = models.Secret"
    # and "db_repo = repo.get_secret_repository()".
    db_model = None
    db_repo = repos.BaseRepo()

    def __init__(self, context=None, **kwargs):
        super(BarbicanObject, self).__init__(context=context, **kwargs)
        self.dict_fields = None
        self.obj_set_defaults()

    def set_attribute(self, name_attr, value_attr=None):
        setattr(self, name_attr, value_attr)

    def _get_db_entity(self):
        return self.db_model(check_exc=False)

    def _get_changed_persistent_fields(self):
        change_fields = self.obj_get_changes()
        for field in self.synthetic_fields:
            if field in change_fields:
                del change_fields[field]
        return change_fields

    def _validate_fields(self, change_fields):
        """Validate fields before creating model

        In order to verify fields before saving a model to database. It should
        be inherited by sub-class in case the class need to verify.
        """
        pass

    def _from_db_object(self, db_entity):
        if db_entity is None:
            return None
        for field in self.fields:
            if field not in self.synthetic_fields:
                self.set_attribute(field, db_entity[field])
        self.load_synthetic_db_fields(db_entity=db_entity)
        self.dict_fields = db_entity.to_dict_fields()
        self.obj_reset_changes()
        return self

    def to_dict_fields(self):
        return self.dict_fields

    def register_value(self, data=None, **kwargs):
        data = data or dict()
        data.update(kwargs)
        for key, value in data.items():
            setattr(self, key, value)

    @classmethod
    def is_synthetic(cls, field):
        return field in cls.synthetic_fields

    @classmethod
    def load_object(cls, db_entity):
        obj = cls()
        obj._from_db_object(db_entity=db_entity)
        return obj

    def load_synthetic_db_fields(self, db_entity):
        """Load synthetic database field.

        :param db_entity: Database model
        :return: None
        """
        for field in self.synthetic_fields:
            objclasses = object_base.VersionedObjectRegistry.obj_classes(
            ).get(self.fields[field].objname)

            objclass = objclasses[0]
            synth_db_objs = db_entity.get(field, None)

            # NOTE(namnh): synth_db_objs can be list, dict, empty list
            if isinstance(self.fields[field], fields.DictOfObjectsField):
                dict_entity_object = {key: objclass.load_object(value)
                                      for key, value in synth_db_objs.items()}
                setattr(self, field, dict_entity_object)
            elif isinstance(self.fields[field], fields.ListOfObjectsField):
                entities_object = [objclass.load_object(entity)
                                   for entity in synth_db_objs]
                setattr(self, field, entities_object)
            else:
                # At this moment, this field is an ObjectField.
                entity_object = objclass.load_object(synth_db_objs)
                setattr(self, field, entity_object)
            self.obj_reset_changes([fields])

    def create(self, session=None):
        change_fields = self._get_changed_persistent_fields()
        self._validate_fields(change_fields)
        db_entity = self._get_db_entity()
        db_entity.update(change_fields)
        db_entity = self.db_repo.create_from(db_entity, session=session)
        self._from_db_object(db_entity)

    def save(self, session=None):
        """To update new values to a row in database."""
        change_fields = self._get_changed_persistent_fields()
        self.db_repo.update_from(self.db_model, self.id,
                                 change_fields, session=session)
        self.obj_reset_changes()

    def delete(self, session):
        raise NotImplementedError()

    @classmethod
    def get(cls, entity_id, external_project_id=None,
            force_show_deleted=False,
            suppress_exception=False, session=None):
        """Get an entity or raise if it does not exist"""
        db_entity = cls.db_repo.get(
            entity_id, external_project_id=external_project_id,
            force_show_deleted=force_show_deleted,
            suppress_exception=suppress_exception, session=session)
        if db_entity:
            return cls()._from_db_object(db_entity)
        else:
            return None

    @classmethod
    def get_session(cls, session=None):
        return session or repos.get_session()

    @classmethod
    def delete_entity_by_id(cls, entity_id, external_project_id,
                            session=None):
        cls.db_repo.delete_entity_by_id(
            entity_id, external_project_id, session=session)

    @classmethod
    def get_project_entities(cls, project_id, session=None):
        """Gets entities associated with a given project."""
        entities_db = cls.db_repo.get_project_entities(project_id,
                                                       session=session)
        entities_object = [cls()._from_db_object(entity_db)
                           for entity_db in entities_db] if entities_db else []
        return entities_object

    @classmethod
    def get_count(cls, project_id, session=None):
        """Gets count of entities associated with a given project"""
        return cls.db_repo.get_count(project_id, session=session)

    @classmethod
    def delete_project_entities(cls, project_id,
                                suppress_exception=False,
                                session=None):
        """Deletes entities for a given project."""
        cls.db_repo.delete_project_entities(
            project_id, suppress_exception=suppress_exception, session=session)



#!/usr/bin/env python3
# Copyright 2014 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Barbican Keystone notification listener server.
"""

import eventlet
import os
import sys

# Oslo messaging notification server uses eventlet.
#
# To have remote debugging, thread module needs to be disabled.
# eventlet.monkey_patch(thread=False)
eventlet.monkey_patch()
# Monkey patch the original current_thread to use the up-to-date _active
# global variable. See https://bugs.launchpad.net/bugs/1863021 and
# https://github.com/eventlet/eventlet/issues/592
import __original_module_threading as orig_threading
import threading  # noqa
orig_threading.current_thread.__globals__['_active'] = threading._active


# 'Borrowed' from the Glance project:
# If ../barbican/__init__.py exists, add ../ to Python search path, so that
# it will override what happens to be installed in /usr/(local/)lib/python...
possible_topdir = os.path.normpath(os.path.join(os.path.abspath(sys.argv[0]),
                                   os.pardir,
                                   os.pardir))
if os.path.exists(os.path.join(possible_topdir, 'barbican', '__init__.py')):
    sys.path.insert(0, possible_topdir)


from barbican.common import config
from barbican import queue
from barbican.queue import keystone_listener
from barbican import version

from oslo_log import log
from oslo_service import service


def fail(returncode, e):
    sys.stderr.write("ERROR: {0}\n".format(e))
    sys.exit(returncode)


def main():
    try:
        config.setup_remote_pydev_debug()

        CONF = config.CONF
        CONF(sys.argv[1:], project='barbican',
             version=version.version_info.version_string)

        # Import and configure logging.
        log.setup(CONF, 'barbican')

        LOG = log.getLogger(__name__)
        LOG.info("Booting up Barbican Keystone listener node...")

        # Queuing initialization
        queue.init(CONF)

        if getattr(getattr(CONF, queue.KS_NOTIFICATIONS_GRP_NAME), 'enable'):
            service.launch(
                CONF,
                keystone_listener.MessageServer(CONF),
                restart_method='mutate'
            ).wait()
        else:
            LOG.info("Exiting as Barbican Keystone listener is not enabled...")
    except RuntimeError as e:
        fail(1, e)


if __name__ == '__main__':
    sys.exit(main())



# Copyright 2011 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
A filter middleware that just outputs to logs, for instructive/sample
purposes only.
"""

from microversion_parse import middleware as microversion_middleware

from barbican.api.controllers import versions
from barbican.api import middleware
from barbican.common import utils

LOG = utils.getLogger(__name__)


class MicroversionMiddleware(
        microversion_middleware.MicroversionMiddleware,
        middleware.Middleware):
    def __init__(self, app):
        super(MicroversionMiddleware, self).__init__(
            app, 'key-manager', versions.VERSIONS)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from alembic import script as alembic_script
from oslo_config import cfg

from barbican.common import config
from barbican.common import exception
from barbican.model.migration import commands as migration
from barbican.model import models
from barbican.model import repositories
from barbican.tests import database_utils
from barbican.tests import utils


class WhenCleaningRepositoryPagingParameters(utils.BaseTestCase):

    def setUp(self):
        super(WhenCleaningRepositoryPagingParameters, self).setUp()
        self.CONF = config.CONF
        self.default_limit = self.CONF.default_limit_paging

    def test_parameters_not_assigned(self):
        """The cleaner should use defaults when params are not specified."""
        clean_offset, clean_limit = repositories.clean_paging_values()

        self.assertEqual(0, clean_offset)
        self.assertEqual(self.default_limit, clean_limit)

    def test_limit_as_none(self):
        """When Limit is set to None it should use the default limit."""
        offset = 0
        clean_offset, clean_limit = repositories.clean_paging_values(
            offset_arg=offset,
            limit_arg=None)

        self.assertEqual(offset, clean_offset)
        self.assertEqual(self.default_limit, clean_limit)

    def test_offset_as_none(self):
        """When Offset is set to None it should use an offset of 0."""
        clean_offset, clean_limit = repositories.clean_paging_values(
            offset_arg=None,
            limit_arg=self.default_limit)

        self.assertEqual(0, clean_offset)
        self.assertEqual(self.default_limit, clean_limit)

    def test_limit_as_uncastable_str(self):
        """When Limit cannot be cast to an int, expect the default."""
        clean_offset, clean_limit = repositories.clean_paging_values(
            offset_arg=0,
            limit_arg='boom')
        self.assertEqual(0, clean_offset)
        self.assertEqual(self.default_limit, clean_limit)

    def test_offset_as_uncastable_str(self):
        """When Offset cannot be cast to an int, it should be zero."""
        clean_offset, clean_limit = repositories.clean_paging_values(
            offset_arg='boom',
            limit_arg=self.default_limit)
        self.assertEqual(0, clean_offset)
        self.assertEqual(self.default_limit, clean_limit)

    def test_limit_is_less_than_one(self):
        """Offset should default to 1."""
        limit = -1
        clean_offset, clean_limit = repositories.clean_paging_values(
            offset_arg=1,
            limit_arg=limit)
        self.assertEqual(1, clean_offset)
        self.assertEqual(1, clean_limit)

    def test_limit_is_too_big(self):
        """Limit should max out at configured value."""
        limit = self.CONF.max_limit_paging + 10
        clean_offset, clean_limit = repositories.clean_paging_values(
            offset_arg=1,
            limit_arg=limit)
        self.assertEqual(self.CONF.max_limit_paging, clean_limit)

    def test_offset_is_too_big(self):
        """When Offset exceeds sys.maxsize, it should be zero."""
        clean_offset, clean_limit = repositories.clean_paging_values(
            offset_arg=265613988875874769338781322035779626829233452653394495,
            limit_arg=self.default_limit)
        self.assertEqual(0, clean_offset)
        self.assertEqual(self.default_limit, clean_limit)


class WhenInvokingExceptionMethods(utils.BaseTestCase):

    def setUp(self):
        super(WhenInvokingExceptionMethods, self).setUp()
        self.CONF = config.CONF

        self.entity_id = '123456'
        self.entity_name = 'test_entity'

    def test_should_raise_for_entity_not_found(self):

        exception_result = self.assertRaises(
            exception.NotFound,
            repositories._raise_entity_not_found,
            self.entity_name,
            self.entity_id)

        self.assertEqual(
            "No test_entity found with ID 123456",
            str(exception_result))

    def test_should_raise_for_entity_id_not_found(self):

        exception_result = self.assertRaises(
            exception.NotFound,
            repositories._raise_entity_id_not_found,
            self.entity_id)

        self.assertEqual(
            "Entity ID 123456 not found",
            str(exception_result))

    def test_should_raise_for_no_entities_found(self):

        exception_result = self.assertRaises(
            exception.NotFound,
            repositories._raise_no_entities_found,
            self.entity_name)

        self.assertEqual(
            "No entities of type test_entity found",
            str(exception_result))


class WhenTestingBaseRepository(database_utils.RepositoryTestCase):

    def setUp(self):
        super(WhenTestingBaseRepository, self).setUp()
        self.repo = repositories.BaseRepo()

    def test_should_raise_invalid_create_from_no_entity(self):
        exception_result = self.assertRaises(
            exception.Invalid,
            self.repo.create_from,
            None)

        self.assertEqual(
            "Must supply non-None Entity.",
            str(exception_result))

    def test_should_raise_invalid_create_from_entity_with_id(self):
        entity = models.ModelBase()
        entity.id = '1234'

        exception_result = self.assertRaises(
            exception.Invalid,
            self.repo.create_from,
            entity)

        self.assertEqual(
            "Must supply Entity with id=None (i.e. new entity).",
            str(exception_result))

    def test_should_raise_invalid_do_validate_no_status(self):
        exception_result = self.assertRaises(
            exception.Invalid,
            self.repo._do_validate,
            {})

        self.assertEqual(
            "Entity status is required.",
            str(exception_result))

    def test_should_raise_invalid_do_validate_bad_status(self):
        exception_result = self.assertRaises(
            exception.Invalid,
            self.repo._do_validate,
            dict(status='BOGUS_STATUS'))

        self.assertEqual(
            "Invalid status 'BOGUS_STATUS' for Entity.",
            str(exception_result))


class WhenTestingGetEnginePrivate(utils.BaseTestCase):

    def setUp(self):
        super(WhenTestingGetEnginePrivate, self).setUp()

        repositories.CONF.set_override("connection", "connection",
                                       "database")

    @mock.patch('barbican.model.repositories._create_engine')
    def test_should_raise_value_exception_engine_create_failure(
            self, mock_create_engine):
        engine = mock.MagicMock()
        engine.connect.side_effect = ValueError('Abort!')
        mock_create_engine.return_value = engine

        exception_result = self.assertRaises(
            exception.BarbicanException,
            repositories._get_engine,
            None)

        self.assertEqual(
            'Error configuring registry database with supplied '
            'database connection. Got error: Abort!',
            str(exception_result))
        engine.connect.assert_called_once_with()

    @mock.patch('barbican.model.repositories._create_engine')
    def test_should_complete_with_no_alembic_create_default_configs(
            self, mock_create_engine):

        repositories.CONF.set_override("db_auto_create", False)
        engine = mock.MagicMock()
        mock_create_engine.return_value = engine

        # Invoke method under test.
        repositories._get_engine(None)

        engine.connect.assert_called_once_with()
        mock_create_engine.assert_called_once_with()

    @mock.patch('barbican.model.repositories._create_engine')
    def test_should_complete_with_no_alembic_create_pool_configs(
            self, mock_create_engine):

        repositories.CONF.set_override("db_auto_create", False)
        repositories.CONF.set_override(
            "sql_pool_class", "QueuePool")
        repositories.CONF.set_override("max_pool_size", 22, 'database')
        repositories.CONF.set_override("max_overflow", 11, 'database')

        engine = mock.MagicMock()
        mock_create_engine.return_value = engine

        # Invoke method under test.
        repositories._get_engine(None)

        engine.connect.assert_called_once_with()
        mock_create_engine.assert_called_once_with()


class WhenTestingAutoGenerateTables(utils.BaseTestCase):

    @mock.patch('barbican.model.migration.commands.upgrade')
    def test_should_complete_with_alembic_database_update(
            self, mock_commands_upgrade):

        tables = dict(
            alembic_version='version')  # Mimic tables already created.
        engine = 'engine'

        # Invoke method under test.
        repositories._auto_generate_tables(engine, tables)

        mock_commands_upgrade.assert_called_once_with()


class WhenTestingIsDbConnectionError(utils.BaseTestCase):

    def test_should_return_false_no_error_code_in_args(self):

        args = mock.MagicMock()
        args.find.return_value = -1

        result = repositories.is_db_connection_error(args)

        self.assertFalse(result)

    def test_should_return_true_error_code_found_in_args(self):

        args = mock.MagicMock()
        args.find.return_value = 1

        result = repositories.is_db_connection_error(args)

        self.assertTrue(result)


class WhenTestingMigrations(utils.BaseTestCase):

    def setUp(self):
        super(WhenTestingMigrations, self).setUp()
        repositories.CONF.set_override("connection", "connection", "database")
        self.alembic_config = migration.init_config()
        self.alembic_config.barbican_config = cfg.CONF

    def test_no_downgrade(self):
        script_dir = alembic_script.ScriptDirectory.from_config(
            self.alembic_config)
        versions = [v for v in script_dir.walk_revisions(base='base',
                                                         head='heads')]
        failed_revisions = []
        for version in versions:
            if hasattr(version.module, 'downgrade'):
                failed_revisions.append(version.revision)

        if failed_revisions:
            self.fail('Migrations %s have downgrade' % failed_revisions)


class DummyRepo(repositories.BaseRepo):
    """Repository for the increasing code coverage of unit tests."""
    def get_session(self, session=None):
        return None

    def _do_entity_name(self):
        return "Dummy"

    def _do_build_get_query(self, entity_id, external_project_id, session):
        return None

    def _do_validate(self, values):
        pass

    def _build_get_project_entities_query(self, project_id, session):
        return None


class WhenIncreasingRepositoryTestCoverage(utils.BaseTestCase):

    def test_get_count_should_return_zero(self):
        dummy_repo = DummyRepo()
        count = dummy_repo.get_count('dummy_project_id')
        self.assertEqual(0, count)

    def test_get_project_entities_should_return_empty(self):
        dummy_repo = DummyRepo()
        count = dummy_repo.get_project_entities('dummy_project_id')
        self.assertEqual([], count)



#    Copyright 2018 Fujitsu.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from oslo_versionedobjects import base as object_base

from barbican.common import exception
from barbican import i18n as u
from barbican.model import models
from barbican.model import repositories as repo
from barbican.objects import base
from barbican.objects import fields


@object_base.VersionedObjectRegistry.register
class TransportKey(base.BarbicanObject, base.BarbicanPersistentObject,
                   object_base.VersionedObjectDictCompat):
    fields = {
        'plugin_name': fields.StringField(),
        'transport_key': fields.StringField(),
        'status': fields.StringField(nullable=True, default=base.States.ACTIVE)
    }

    db_model = models.TransportKey
    db_repo = repo.get_transport_key_repository()

    def _validate_fields(self, change_fields):
        msg = u._("Must supply non-None {0} argument for TransportKey entry.")

        if change_fields.get('plugin_name') is None:
            raise exception.MissingArgumentError(msg.format("plugin_name"))

        if change_fields.get('transport_key') is None:
            raise exception.MissingArgumentError(msg.format("transport_key"))

    @classmethod
    def get_by_create_date(cls, plugin_name=None,
                           offset_arg=None, limit_arg=None,
                           suppress_exception=False, session=None):
        transport_keys_db, offset, limit, total = \
            cls.db_repo.get_by_create_date(plugin_name, offset_arg, limit_arg,
                                           suppress_exception, session)
        transport_keys_obj = [cls()._from_db_object(transport_key)
                              for transport_key in transport_keys_db]
        return transport_keys_obj, offset, limit, total

    @classmethod
    def get_latest_transport_key(cls, plugin_name, suppress_exception=False,
                                 session=None):
        transport_key_db = cls.db_repo.get_latest_transport_key(
            plugin_name, suppress_exception, session)
        return cls()._from_db_object(transport_key_db)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os

from cryptography import fernet
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.asymmetric import dsa
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.primitives import serialization
from oslo_config import cfg
from oslo_utils import encodeutils

from barbican.common import config
from barbican.common import utils
from barbican import i18n as u
from barbican.plugin.crypto import base as c


CONF = config.new_config()
LOG = utils.getLogger(__name__)

simple_crypto_plugin_group = cfg.OptGroup(name='simple_crypto_plugin',
                                          title="Simple Crypto Plugin Options")
simple_crypto_plugin_opts = [
    cfg.MultiStrOpt(
        'kek',
        default=['dGhpcnR5X3R3b19ieXRlX2tleWJsYWhibGFoYmxhaGg='],
        secret=True,
        help=u._('Fernet Key-Encryption Key (KEK) to be used by SimpleCrypto '
                 'Plugin to encrypt Project-specific KEKs.'),
    ),
    cfg.StrOpt('plugin_name',
               help=u._('User friendly plugin name'),
               default='Software Only Crypto'),
]
CONF.register_group(simple_crypto_plugin_group)
CONF.register_opts(simple_crypto_plugin_opts, group=simple_crypto_plugin_group)
config.parse_args(CONF)


def list_opts():
    yield simple_crypto_plugin_group, simple_crypto_plugin_opts


class SimpleCryptoPlugin(c.CryptoPluginBase):
    """Insecure implementation of the crypto plugin."""

    def __init__(self, conf=CONF):
        if len(conf.simple_crypto_plugin.kek) < 1:
            raise ValueError(u._("SimpleCrypto KEK is undefined"))
        self.master_keys = conf.simple_crypto_plugin.kek
        self.plugin_name = conf.simple_crypto_plugin.plugin_name
        LOG.info("{} initialized".format(self.plugin_name))

    def get_plugin_name(self):
        return self.plugin_name

    def _get_kek(self, kek_meta_dto):
        if not kek_meta_dto.plugin_meta:
            raise ValueError(u._('KEK not yet created.'))
        # the kek is stored encrypted. Need to decrypt.
        encryptor = fernet.MultiFernet(
            [fernet.Fernet(x) for x in self.master_keys]
        )
        # Note : If plugin_meta type is unicode, encode to byte.
        if isinstance(kek_meta_dto.plugin_meta, str):
            kek_meta_dto.plugin_meta = kek_meta_dto.plugin_meta.encode('utf-8')

        return encryptor.decrypt(kek_meta_dto.plugin_meta)

    def encrypt(self, encrypt_dto, kek_meta_dto, project_id):
        kek = self._get_kek(kek_meta_dto)
        unencrypted = encrypt_dto.unencrypted
        if not isinstance(unencrypted, bytes):
            raise ValueError(
                u._(
                    'Unencrypted data must be a byte type, but was '
                    '{unencrypted_type}'
                ).format(
                    unencrypted_type=type(unencrypted)
                )
            )
        encryptor = fernet.Fernet(kek)
        cyphertext = encryptor.encrypt(unencrypted)
        return c.ResponseDTO(cyphertext, None)

    def decrypt(self, encrypted_dto, kek_meta_dto, kek_meta_extended,
                project_id):
        kek = self._get_kek(kek_meta_dto)
        encrypted = encrypted_dto.encrypted
        decryptor = fernet.Fernet(kek)
        return decryptor.decrypt(encrypted)

    def bind_kek_metadata(self, kek_meta_dto):
        kek_meta_dto.algorithm = 'aes'
        kek_meta_dto.bit_length = 128
        kek_meta_dto.mode = 'cbc'
        if not kek_meta_dto.plugin_meta:
            # the kek is stored encrypted in the plugin_meta field
            encryptor = fernet.Fernet(self.master_keys[0])
            key = fernet.Fernet.generate_key()
            kek_meta_dto.plugin_meta = encryptor.encrypt(key)
        return kek_meta_dto

    def generate_symmetric(self, generate_dto, kek_meta_dto, project_id):
        byte_length = int(generate_dto.bit_length) // 8
        unencrypted = os.urandom(byte_length)

        return self.encrypt(c.EncryptDTO(unencrypted),
                            kek_meta_dto,
                            project_id)

    def generate_asymmetric(self, generate_dto, kek_meta_dto, project_id):
        """Generate asymmetric keys based on below rules:

        - RSA, with passphrase (supported)
        - RSA, without passphrase (supported)
        - DSA, without passphrase (supported)
        - DSA, with passphrase (supported)
        """
        if (generate_dto.algorithm is None or generate_dto
                .algorithm.lower() == 'rsa'):
            private_key = rsa.generate_private_key(
                public_exponent=65537,
                key_size=generate_dto.bit_length,
                backend=default_backend()
            )
        elif generate_dto.algorithm.lower() == 'dsa':
            private_key = dsa.generate_private_key(
                key_size=generate_dto.bit_length,
                backend=default_backend()
            )
        else:
            raise c.CryptoPrivateKeyFailureException()

        public_key = private_key.public_key()

        if generate_dto.algorithm.lower() == 'rsa':
            private_key = private_key.private_bytes(
                encoding=serialization.Encoding.PEM,
                format=serialization.PrivateFormat.PKCS8,
                encryption_algorithm=self._get_encryption_algorithm(
                    generate_dto.passphrase)
            )

            public_key = public_key.public_bytes(
                encoding=serialization.Encoding.PEM,
                format=serialization.PublicFormat.SubjectPublicKeyInfo
            )

        if generate_dto.algorithm.lower() == 'dsa':
            private_key = private_key.private_bytes(
                encoding=serialization.Encoding.DER,
                format=serialization.PrivateFormat.PKCS8,
                encryption_algorithm=self._get_encryption_algorithm(
                    generate_dto.passphrase)
            )
            public_key = public_key.public_bytes(
                encoding=serialization.Encoding.DER,
                format=serialization.PublicFormat.SubjectPublicKeyInfo
            )

        private_dto = self.encrypt(c.EncryptDTO(private_key),
                                   kek_meta_dto,
                                   project_id)

        public_dto = self.encrypt(c.EncryptDTO(public_key),
                                  kek_meta_dto,
                                  project_id)

        passphrase_dto = None
        if generate_dto.passphrase:
            if isinstance(generate_dto.passphrase, str):
                generate_dto.passphrase = generate_dto.passphrase.encode(
                    'utf-8')

            passphrase_dto = self.encrypt(c.EncryptDTO(generate_dto.
                                                       passphrase),
                                          kek_meta_dto,
                                          project_id)

        return private_dto, public_dto, passphrase_dto

    def supports(self, type_enum, algorithm=None, bit_length=None,
                 mode=None):
        if type_enum == c.PluginSupportTypes.ENCRYPT_DECRYPT:
            return True
        elif type_enum == c.PluginSupportTypes.SYMMETRIC_KEY_GENERATION:
            return self._is_algorithm_supported(algorithm,
                                                bit_length,
                                                mode)
        elif type_enum == c.PluginSupportTypes.ASYMMETRIC_KEY_GENERATION:
            return self._is_algorithm_supported(algorithm,
                                                bit_length,
                                                mode)
        else:
            return False

    def _get_encryption_algorithm(self, passphrase):
        """Choose whether to use encryption or not based on passphrase

        serialization.BestAvailableEncryption fails if passphrase is not
        given or if less than one byte therefore we need to check if it is
        valid or not
        """
        if passphrase:
            # encryption requires password in bytes format
            algorithm = serialization.BestAvailableEncryption(
                # default encoding is utf-8
                encodeutils.safe_encode(passphrase)
            )
        else:
            algorithm = serialization.NoEncryption()

        return algorithm

    def _is_algorithm_supported(self, algorithm=None,
                                bit_length=None, mode=None):
        """check if algorithm and bit_length combination is supported."""
        if algorithm is None or bit_length is None:
            return False

        length_factor = 1

        # xts-mode cuts the effective key for the algorithm in half,
        # so the bit_length must be the double of the supported length.
        # in the future there should be a validation of supported modes too.
        if mode is not None and mode.lower() == "xts":
            length_factor = 2

        if (algorithm.lower() in c.PluginSupportTypes.SYMMETRIC_ALGORITHMS
            and bit_length / length_factor
                in c.PluginSupportTypes.SYMMETRIC_KEY_LENGTHS):
            return True
        elif (algorithm.lower() in c.PluginSupportTypes.ASYMMETRIC_ALGORITHMS
              and bit_length in c.PluginSupportTypes.ASYMMETRIC_KEY_LENGTHS):
            return True
        else:
            return False



# Copyright (c) 2013-2014 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from contextlib import contextmanager
import datetime
import functools
import os
from os import path
import time
import types
from unittest import mock

from oslo_config import cfg
from oslo_utils import uuidutils
import oslotest.base as oslotest
from oslotest import createfile
from urllib import parse
import webtest

from barbican.api import app
from barbican.common import config
import barbican.context
from barbican.model import repositories
from barbican.plugin.crypto import manager as cm
from barbican.plugin.crypto import p11_crypto
from barbican.plugin.interface import secret_store
from barbican.plugin import kmip_secret_store as kss
from barbican.tests import database_utils
from barbican.tests import fixture as barbican_fixture


def mock_pecan_request(test_instance, host=None, version='1.0'):
    patcher_obj = mock.patch('pecan.request')
    mock_req = patcher_obj.start()
    test_instance.addCleanup(patcher_obj.stop)
    mock_req.url = host
    mock_req.environ = os.environ.copy()
    mock_req.environ['key-manager.microversion'] = version
    mock_req.application_url = host


@contextmanager
def pecan_context(test_instance, host=None):
    mock_pecan_request(test_instance, host=host)
    yield


class BarbicanAPIBaseTestCase(oslotest.BaseTestCase):
    """Base TestCase for all tests needing to interact with a Barbican app."""
    root_controller = None

    def _build_context(
        self, project_id, roles=None, user_id=None, is_admin=True,
        policy_enforcer=None,
    ):
        context = barbican.context.RequestContext(
            roles=roles,
            user_id=user_id,
            project_id=project_id,
            is_admin=is_admin
        )
        context.policy_enforcer = policy_enforcer
        return context

    def setUp(self):
        super().setUp()
        self.useFixture(barbican_fixture.StandardLogging())
        self.useFixture(barbican_fixture.WarningsFixture())
        # Make sure we have a test db and session to work with
        database_utils.setup_in_memory_db()

        # Generic project id to perform actions under
        self.project_id = generate_test_valid_uuid()

        # Build the test app
        wsgi_app = app.build_wsgi_app(
            controller=self.root_controller,
            transactional=True
        )

        self.app = webtest.TestApp(wsgi_app)
        self.app.extra_environ = {
            'barbican.context': self._build_context(self.project_id)
        }

    def tearDown(self):
        database_utils.in_memory_cleanup()
        super().tearDown()


class BaseTestCase(oslotest.BaseTestCase):
    def setUp(self):
        super().setUp()
        self.useFixture(barbican_fixture.StandardLogging())
        self.useFixture(barbican_fixture.WarningsFixture())
        self.order_id = 'order1234'
        self.external_project_id = 'keystone1234'
        self.request_id = 'request1234'

    def tearDown(self):
        ss_conf = config.get_module_config('secretstore')
        ss_conf.clear_override("enable_multiple_secret_stores",
                               group='secretstore')
        super().tearDown()


class MockModelRepositoryMixin(object):
    """Class for setting up the repo factory mocks

    This class has the purpose of setting up the mocks for the model repository
    factory functions. This is because they are intended to be singletons, and
    thus called inside the code-base, and not really passed around as
    arguments. Thus, this kind of approach is needed.

    The functions assume that the class that inherits from this is a test case
    fixture class. This is because as a side-effect patcher objects will be
    added to the class, and also the cleanup of these patcher objects will be
    added to the tear-down of the respective classes.
    """

    def setup_container_consumer_repository_mock(
            self, mock_container_consumer_repo=mock.MagicMock()):
        """Mocks the container consumer repository factory function

        :param mock_container_consumer_repo: The pre-configured mock
                                             container consumer repo to be
                                             returned.
        """
        self.mock_container_consumer_repo_patcher = None
        self._setup_repository_mock(
            repo_factory='get_container_consumer_repository',
            mock_repo_obj=mock_container_consumer_repo,
            patcher_obj=self.mock_container_consumer_repo_patcher)

    def setup_secret_consumer_repository_mock(
            self, mock_secret_consumer_repo=mock.MagicMock()):
        """Mocks the secret consumer repository factory function

        :param mock_secret_consumer_repo: The pre-configured mock
                                          secret consumer repo to be
                                          returned.
        """
        self.mock_secret_consumer_repo_patcher = None
        self._setup_repository_mock(
            repo_factory='get_secret_consumer_repository',
            mock_repo_obj=mock_secret_consumer_repo,
            patcher_obj=self.mock_secret_consumer_repo_patcher)

    def setup_container_repository_mock(self,
                                        mock_container_repo=mock.MagicMock()):
        """Mocks the container repository factory function

        :param mock_container_repo: The pre-configured mock
                                    container repo to be returned.
        """
        self.mock_container_repo_patcher = None
        self._setup_repository_mock(
            repo_factory='get_container_repository',
            mock_repo_obj=mock_container_repo,
            patcher_obj=self.mock_container_repo_patcher)

    def setup_container_secret_repository_mock(
            self, mock_container_secret_repo=mock.MagicMock()):
        """Mocks the container-secret repository factory function

        :param mock_container_secret_repo: The pre-configured mock
                                           container-secret repo to be
                                           returned.
        """
        self.mock_container_secret_repo_patcher = None
        self._setup_repository_mock(
            repo_factory='get_container_secret_repository',
            mock_repo_obj=mock_container_secret_repo,
            patcher_obj=self.mock_container_secret_repo_patcher)

    def setup_encrypted_datum_repository_mock(
            self, mock_encrypted_datum_repo=mock.MagicMock()):
        """Mocks the encrypted datum repository factory function

        :param mock_encrypted_datum_repo: The pre-configured mock
                                          encrypted datum repo to be returned.
        """
        self.mock_encrypted_datum_repo_patcher = None
        self._setup_repository_mock(
            repo_factory='get_encrypted_datum_repository',
            mock_repo_obj=mock_encrypted_datum_repo,
            patcher_obj=self.mock_encrypted_datum_repo_patcher)

    def setup_kek_datum_repository_mock(self,
                                        mock_kek_datum_repo=mock.MagicMock()):
        """Mocks the kek datum repository factory function

        :param mock_kek_datum_repo: The pre-configured mock kek-datum repo to
                                    be returned.
        """
        self.mock_kek_datum_repo_patcher = None
        self._setup_repository_mock(
            repo_factory='get_kek_datum_repository',
            mock_repo_obj=mock_kek_datum_repo,
            patcher_obj=self.mock_kek_datum_repo_patcher)

    def setup_order_barbican_meta_repository_mock(
            self, mock_order_barbican_meta_repo=mock.MagicMock()):
        """Mocks the order-barbican-meta repository factory function

        :param mock_order_barbican_meta_repo: The pre-configured mock order
                                              barbican-meta repo to be
                                              returned.
        """
        self.mock_order_barbican_meta_repo_patcher = None
        self._setup_repository_mock(
            repo_factory='get_order_barbican_meta_repository',
            mock_repo_obj=mock_order_barbican_meta_repo,
            patcher_obj=self.mock_order_barbican_meta_repo_patcher)

    def setup_order_plugin_meta_repository_mock(
            self, mock_order_plugin_meta_repo=mock.MagicMock()):
        """Mocks the order-plugin-meta repository factory function

        :param mock_order_plugin_meta_repo: The pre-configured mock order
                                            plugin-meta repo to be returned.
        """
        self.mock_order_plugin_meta_repo_patcher = None
        self._setup_repository_mock(
            repo_factory='get_order_plugin_meta_repository',
            mock_repo_obj=mock_order_plugin_meta_repo,
            patcher_obj=self.mock_order_plugin_meta_repo_patcher)

    def setup_order_repository_mock(self, mock_order_repo=mock.MagicMock()):
        """Mocks the order repository factory function

        :param mock_order_repo: The pre-configured mock order repo to be
                                returned.
        """
        self.mock_order_repo_patcher = None
        self._setup_repository_mock(repo_factory='get_order_repository',
                                    mock_repo_obj=mock_order_repo,
                                    patcher_obj=self.mock_order_repo_patcher)

    def setup_project_repository_mock(self,
                                      mock_project_repo=mock.MagicMock()):
        """Mocks the project repository factory function

        :param mock_project_repo: The pre-configured mock project repo to be
                                  returned.
        """
        self.mock_project_repo_patcher = None
        self._setup_repository_mock(repo_factory='get_project_repository',
                                    mock_repo_obj=mock_project_repo,
                                    patcher_obj=self.mock_project_repo_patcher)

    def setup_secret_meta_repository_mock(
            self, mock_secret_meta_repo=mock.MagicMock()):
        """Mocks the secret-meta repository factory function

        :param mock_secret_meta_repo: The pre-configured mock secret-meta repo
                                      to be returned.
        """
        self.mock_secret_meta_repo_patcher = None
        self._setup_repository_mock(
            repo_factory='get_secret_meta_repository',
            mock_repo_obj=mock_secret_meta_repo,
            patcher_obj=self.mock_secret_meta_repo_patcher)

    def setup_secret_repository_mock(self, mock_secret_repo=mock.MagicMock()):
        """Mocks the secret repository factory function

        :param mock_secret_repo: The pre-configured mock secret repo to be
                                 returned.
        """
        self.mock_secret_repo_patcher = None
        self._setup_repository_mock(repo_factory='get_secret_repository',
                                    mock_repo_obj=mock_secret_repo,
                                    patcher_obj=self.mock_secret_repo_patcher)

    def setup_transport_key_repository_mock(
            self, mock_transport_key_repo=mock.MagicMock()):
        """Mocks the transport-key repository factory function

        :param mock_transport_key_repo: The pre-configured mock transport_key
                                        repo to be returned.
        """
        self.mock_transport_key_repo_patcher = None
        self._setup_repository_mock(
            repo_factory='get_transport_key_repository',
            mock_repo_obj=mock_transport_key_repo,
            patcher_obj=self.mock_transport_key_repo_patcher)

    def setup_ca_repository_mock(self, mock_ca_repo=mock.MagicMock()):
        """Mocks the project repository factory function

        :param mock_ca_repo: The pre-configured mock ca repo to be returned.
        """
        self.mock_ca_repo_patcher = None
        self._setup_repository_mock(repo_factory='get_ca_repository',
                                    mock_repo_obj=mock_ca_repo,
                                    patcher_obj=self.mock_ca_repo_patcher)

    def setup_preferred_ca_repository_mock(
            self, mock_preferred_ca_repo=mock.MagicMock()):
        """Mocks the project repository factory function

        :param mock_preferred_ca_repo: The pre-configured mock project ca repo
                                     to be returned.
        """
        self.mock_preferred_ca_repo_patcher = None
        self._setup_repository_mock(
            repo_factory='get_preferred_ca_repository',
            mock_repo_obj=mock_preferred_ca_repo,
            patcher_obj=self.mock_preferred_ca_repo_patcher)

    def setup_secret_stores_repository_mock(
            self, mock_secret_stores_repo=mock.MagicMock()):
        """Mocks the project repository factory function

        :param mock_secret_stores_repo: The pre-configured mock secret stores
                                     repo to be returned.
        """
        self.mock_secret_stores_repo_patcher = None
        self._setup_repository_mock(
            repo_factory='get_secret_stores_repository',
            mock_repo_obj=mock_secret_stores_repo,
            patcher_obj=self.mock_secret_stores_repo_patcher)

    def setup_project_secret_store_repository_mock(
            self, mock_project_secret_store_repo=mock.MagicMock()):
        """Mocks the project repository factory function

        :param mock_project_secret_store_repo: The pre-configured mock project
                                     secret store repo to be returned.
        """
        self.mock_proj_secret_store_repo_patcher = None
        self._setup_repository_mock(
            repo_factory='get_project_secret_store_repository',
            mock_repo_obj=mock_project_secret_store_repo,
            patcher_obj=self.mock_proj_secret_store_repo_patcher)

    def setup_project_ca_repository_mock(
            self, mock_project_ca_repo=mock.MagicMock()):
        """Mocks the project repository factory function

        :param mock_project_ca_repo: The pre-configured mock project ca repo
                                     to be returned.
        """
        self.mock_project_ca_repo_patcher = None
        self._setup_repository_mock(
            repo_factory='get_project_ca_repository',
            mock_repo_obj=mock_project_ca_repo,
            patcher_obj=self.mock_project_ca_repo_patcher)

    def _setup_repository_mock(self, repo_factory, mock_repo_obj, patcher_obj):
        patcher_obj = mock.patch(
            'barbican.model.repositories.' + repo_factory,
            return_value=mock_repo_obj
        )
        patcher_obj.start()
        self.addCleanup(patcher_obj.stop)


def construct_new_test_function(original_func, name, build_params):
    """Builds a new test function based on parameterized data.

    :param original_func: The original test function that is used as a template
    :param name: The fullname of the new test function
    :param build_params: A dictionary or list containing args or kwargs
        for the new test
    :return: A new function object
    """
    new_func = types.FunctionType(
        original_func.__code__,
        original_func.__globals__,
        name=name,
        argdefs=original_func.__defaults__,
        closure=original_func.__closure__
    )

    for key, val in original_func.__dict__.items():
        if key != 'build_data':
            new_func.__dict__[key] = val

    # Support either an arg list or kwarg dict for our data
    build_args = build_params if isinstance(build_params, list) else []
    build_kwargs = build_params if isinstance(build_params, dict) else {}

    # Build a test wrapper to execute with our kwargs
    def test_wrapper(func, test_args, test_kwargs):
        @functools.wraps(func)
        def wrapper(self):
            return func(self, *test_args, **test_kwargs)
        return wrapper

    return test_wrapper(new_func, build_args, build_kwargs)


def process_parameterized_function(name, func_obj, build_data):
    """Build lists of functions to add and remove to a test case."""
    to_remove = []
    to_add = []

    for subtest_name, params in build_data.items():
        # Build new test function
        func_name = '{0}_{1}'.format(name, subtest_name)
        new_func = construct_new_test_function(func_obj, func_name, params)

        # Mark the new function as needed to be added to the class
        to_add.append((func_name, new_func))

        # Mark key for removal
        to_remove.append(name)

    return to_remove, to_add


def parameterized_test_case(cls):
    """Class decorator to process parameterized tests

    This allows for parameterization to be used for potentially any
    unittest compatible runner; including testr and py.test.
    """
    tests_to_remove = []
    tests_to_add = []
    for key, val in vars(cls).items():
        # Only process tests with build data on them
        if key.startswith('test_') and val.__dict__.get('build_data'):
            to_remove, to_add = process_parameterized_function(
                name=key,
                func_obj=val,
                build_data=val.__dict__.get('build_data')
            )
            tests_to_remove.extend(to_remove)
            tests_to_add.extend(to_add)

    # Add all new test functions
    [setattr(cls, name, func) for name, func in tests_to_add]

    # Remove all old test function templates (if they still exist)
    [delattr(cls, key) for key in tests_to_remove if hasattr(cls, key)]
    return cls


def parameterized_dataset(build_data):
    """Simple decorator to mark a test method for processing."""
    def decorator(func):
        func.__dict__['build_data'] = build_data
        return func
    return decorator


def setup_oslo_config_conf(testcase, content, conf_instance=None):

    conf_file_fixture = testcase.useFixture(
        createfile.CreateFileWithContent('barbican', content))
    if conf_instance is None:
        conf_instance = cfg.CONF
    conf_instance([], project="barbican",
                  default_config_files=[conf_file_fixture.path])

    testcase.addCleanup(conf_instance.reset)


def setup_multiple_secret_store_plugins_conf(testcase, store_plugin_names,
                                             crypto_plugin_names,
                                             global_default_index,
                                             conf_instance=None,
                                             multiple_support_enabled=None):
    """Sets multiple secret store support conf as oslo conf file.

    Generating file based conf based on input store and crypto plugin names
    provided as list. Index specified in argument is used to mark that specific
    secret store as global_default = True.

    Input lists are
    'store_plugins': ['store_crypto', 'kmip_plugin', 'store_crypto'],
    'crypto_plugins': ['simple_crypto', '', 'p11_crypto'],

    Sample output conf file generated is

    [secretstore]
    enable_multiple_secret_stores = True
    stores_lookup_suffix = plugin_0, plugin_1, plugin_2

    [secretstore:plugin_0]
    secret_store_plugin = store_crypto
    crypto_plugin = simple_crypto
    global_default = True

    [secretstore:plugin_1]
    secret_store_plugin = kmip_plugin

    [secretstore:plugin_2]
    secret_store_plugin = store_crypto
    crypto_plugin = p11_crypto

    """

    def _get_conf_line(name, value, section=None):
        out_line = "\n[{0}]\n".format(section) if section else ""
        out_line += "{0} = {1}\n".format(name, value) if name else ""
        return out_line

    if multiple_support_enabled is None:
        multiple_support_enabled = True

    conf_content = ""

    if store_plugin_names is not None:

        if len(store_plugin_names) < len(crypto_plugin_names):
            max_count = len(crypto_plugin_names)
        else:
            max_count = len(store_plugin_names)

        lookup_names = ['plugin_{0}'.format(indx) for indx in range(max_count)]
        section_names = ['secretstore:{0}'.format(lname) for lname in
                         lookup_names]
        lookup_str = ", ".join(lookup_names)

        conf_content = _get_conf_line('enable_multiple_secret_stores',
                                      multiple_support_enabled,
                                      section='secretstore')
        conf_content += _get_conf_line('stores_lookup_suffix',
                                       lookup_str, section=None)

        for indx, section_name in enumerate(section_names):
            if indx < len(store_plugin_names):
                store_plugin = store_plugin_names[indx]
                conf_content += _get_conf_line('secret_store_plugin',
                                               store_plugin,
                                               section=section_name)
            else:
                conf_content += _get_conf_line(None, None,
                                               section=section_name)
            if indx < len(crypto_plugin_names):
                crypto_plugin = crypto_plugin_names[indx]
                conf_content += _get_conf_line('crypto_plugin', crypto_plugin,
                                               section=None)
            if indx == global_default_index:
                conf_content += _get_conf_line('global_default', 'True',
                                               section=None)

    setup_oslo_config_conf(testcase, conf_content, conf_instance)


class MultipleBackendsTestCase(database_utils.RepositoryTestCase):

    def _mock_plugin_settings(self):

        kmip_conf = kss.CONF
        kmip_conf.kmip_plugin.username = "sample_username"
        kmip_conf.kmip_plugin.password = "sample_password"
        kmip_conf.kmip_plugin.keyfile = None
        kmip_conf.kmip_plugin.pkcs1_only = False

        pkcs11_conf = p11_crypto.CONF
        pkcs11_conf.p11_crypto_plugin.library_path = "/tmp"  # any dummy path

    def init_via_conf_file(self, store_plugin_names, crypto_plugin_names,
                           enabled=True, global_default_index=0):
        secretstore_conf = config.get_module_config('secretstore')

        setup_multiple_secret_store_plugins_conf(
            self, store_plugin_names=store_plugin_names,
            crypto_plugin_names=crypto_plugin_names,
            global_default_index=global_default_index,
            conf_instance=secretstore_conf,
            multiple_support_enabled=enabled)

        # clear globals if already set in previous tests
        secret_store._SECRET_STORE = None  # clear secret store manager
        cm._PLUGIN_MANAGER = None  # clear crypto manager
        self._mock_plugin_settings()

    def _get_secret_store_entry(self, store_plugin, crypto_plugin):
        all_ss = repositories.get_secret_stores_repository().get_all()
        for ss in all_ss:
            if (ss.store_plugin == store_plugin and
                    ss.crypto_plugin == crypto_plugin):
                return ss
        return None


def create_timestamp_w_tz_and_offset(timezone=None, days=0, hours=0, minutes=0,
                                     seconds=0):
    """Creates a timestamp with a timezone and offset in days

    :param timezone: Timezone used in creation of timestamp
    :param days: The offset in days
    :param hours: The offset in hours
    :param minutes: The offset in minutes

    :return: a timestamp
    """
    if timezone is None:
        timezone = time.strftime("%z")

    timestamp = '{time}{timezone}'.format(
        time=(datetime.datetime.today() + datetime.timedelta(days=days,
                                                             hours=hours,
                                                             minutes=minutes,
                                                             seconds=seconds)),
        timezone=timezone)

    return timestamp


def get_limit_and_offset_from_ref(ref):
    matches = dict(parse.parse_qsl(parse.urlparse(ref).query))
    ref_limit = matches['limit']
    ref_offset = matches['offset']

    return ref_limit, ref_offset


def get_tomorrow_timestamp():
    tomorrow = (datetime.today() + datetime.timedelta(days=1))
    return tomorrow.isoformat()


def get_id_from_ref(ref):
    """Returns id from reference."""
    ref_id = None
    if ref is not None and len(ref) > 0:
        ref_id = path.split(ref)[1]
    return ref_id


def generate_test_uuid(tail_value=0):
    """Returns a blank uuid with the given value added to the end segment."""
    return '00000000-0000-0000-0000-{value:0>{pad}}'.format(value=tail_value,
                                                            pad=12)


def generate_test_valid_uuid():
    """Returns a valid uuid value, similar to uuid generated in barbican"""
    return uuidutils.generate_uuid()


def get_symmetric_key():
    s = b"MIICdgIBADANBgkqhkiG9w=="
    return s


def get_triple_des_key():
    s = b"AQIDBAUGBwgBAgMEBQYHCAECAwQFBgcI"
    return s


def is_kmip_enabled():
    return os.environ.get('KMIP_PLUGIN_ENABLED') is not None


def is_vault_enabled():
    return os.environ.get('VAULT_PLUGIN_ENABLED') is not None


def is_pkcs11_enabled():
    return os.environ.get('PKCS11_PLUGIN_ENABLED') is not None


class DummyClassForTesting(object):
    pass


def set_version(app, version):
    """Sets the requested version in the environ"""
    app.extra_environ['key-manager.microversion'] = version



#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from oslo_log import versionutils
from oslo_policy import policy

from barbican.common.policies import base


deprecated_secret_meta_get = policy.DeprecatedRule(
    name='secret_meta:get',
    check_str='rule:secret_non_private_read or ' +
              'rule:secret_project_creator or ' +
              'rule:secret_project_admin or rule:secret_acl_read',
    deprecated_reason=base.LEGACY_POLICY_DEPRECATION,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_secret_meta_post = policy.DeprecatedRule(
    name='secret_meta:post',
    check_str='rule:secret_project_admin or ' +
              'rule:secret_project_creator or ' +
              '(rule:secret_project_creator_role and ' +
              'rule:secret_non_private_read)',
    deprecated_reason=base.LEGACY_POLICY_DEPRECATION,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_secret_meta_put = policy.DeprecatedRule(
    name='secret_meta:put',
    check_str='rule:secret_project_admin or ' +
              'rule:secret_project_creator or ' +
              '(rule:secret_project_creator_role and ' +
              'rule:secret_non_private_read)',
    deprecated_reason=base.LEGACY_POLICY_DEPRECATION,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_secret_meta_delete = policy.DeprecatedRule(
    name='secret_meta:delete',
    check_str='rule:secret_project_admin or ' +
              'rule:secret_project_creator or ' +
              '(rule:secret_project_creator_role and ' +
              'rule:secret_non_private_read)',
    deprecated_reason=base.LEGACY_POLICY_DEPRECATION,
    deprecated_since=versionutils.deprecated.WALLABY
)

rules = [
    policy.DocumentedRuleDefault(
        name='secret_meta:get',
        check_str=(
            "True:%(enforce_new_defaults)s and "
            "(rule:secret_project_admin or "
            "(rule:secret_project_member and rule:secret_owner) or "
            "(rule:secret_project_member and rule:secret_is_not_private) or "
            "rule:secret_acl_read)"),
        scope_types=['project'],
        description='metadata/: Lists a secrets user-defined metadata. || ' +
                    'metadata/{key}: Retrieves a secrets user-added metadata.',
        operations=[
            {
                'path': '/v1/secrets/{secret-id}/metadata',
                'method': 'GET'
            },
            {
                'path': '/v1/secrets/{secret-id}/metadata/{meta-key}',
                'method': 'GET'
            }
        ],
        deprecated_rule=deprecated_secret_meta_get
    ),
    policy.DocumentedRuleDefault(
        name='secret_meta:post',
        check_str=(
            "True:%(enforce_new_defaults)s and "
            "(rule:secret_project_admin or "
            "(rule:secret_project_member and rule:secret_owner) or "
            "(rule:secret_project_member and rule:secret_is_not_private))"),
        scope_types=['project'],
        description='Adds a new key/value pair to the secrets user-defined ' +
                    'metadata.',
        operations=[
            {
                'path': '/v1/secrets/{secret-id}/metadata/{meta-key}',
                'method': 'POST'
            }
        ],
        deprecated_rule=deprecated_secret_meta_post
    ),
    policy.DocumentedRuleDefault(
        name='secret_meta:put',
        check_str=(
            "True:%(enforce_new_defaults)s and "
            "(rule:secret_project_admin or "
            "(rule:secret_project_member and rule:secret_owner) or "
            "(rule:secret_project_member and rule:secret_is_not_private))"),
        scope_types=['project'],
        description='metadata/: Sets the user-defined metadata for a secret ' +
                    '|| metadata/{key}: Updates an existing key/value pair ' +
                    'in the secrets user-defined metadata.',
        operations=[
            {
                'path': '/v1/secrets/{secret-id}/metadata',
                'method': 'PUT'
            },
            {
                'path': '/v1/secrets/{secret-id}/metadata/{meta-key}',
                'method': 'PUT'
            }
        ],
        deprecated_rule=deprecated_secret_meta_put
    ),
    policy.DocumentedRuleDefault(
        name='secret_meta:delete',
        check_str=(
            "True:%(enforce_new_defaults)s and "
            "(rule:secret_project_admin or "
            "(rule:secret_project_member and rule:secret_owner) or "
            "(rule:secret_project_member and rule:secret_is_not_private))"),
        scope_types=['project'],
        description='Delete secret user-defined metadata by key.',
        operations=[
            {
                'path': '/v1/secrets/{secret-id}/metadata/{meta-key}',
                'method': 'DELETE'
            }
        ],
        deprecated_rule=deprecated_secret_meta_delete
    ),
]


def list_rules():
    return rules



# (c) Copyright 2015-2016 Hewlett Packard Enterprise Development LP
#
#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from unittest import mock

from oslo_utils import uuidutils

from barbican.model import models
from barbican.model import repositories as repos
from barbican.plugin.interface import secret_store
from barbican.tests import utils


class SecretStoresMixin(utils.MultipleBackendsTestCase):

    def _create_project(self):
        session = repos.get_project_repository().get_session()

        project = models.Project()
        project.external_id = ("keystone_project_id" +
                               uuidutils.generate_uuid(dashed=False))
        project.save(session=session)
        return project

    def _create_project_store(self, project_id, secret_store_id):
        proj_store_repo = repos.get_project_secret_store_repository()
        session = proj_store_repo.get_session()

        proj_model = models.ProjectSecretStore(project_id, secret_store_id)

        proj_s_store = proj_store_repo.create_from(proj_model, session)
        proj_s_store.save(session=session)
        return proj_s_store

    def _init_multiple_backends(self, enabled=True, global_default_index=0):

        store_plugin_names = ['store_crypto', 'kmip_plugin', 'store_crypto']
        crypto_plugin_names = ['p11_crypto', '', 'simple_crypto']

        self.init_via_conf_file(store_plugin_names,
                                crypto_plugin_names, enabled=enabled,
                                global_default_index=global_default_index)

        with mock.patch('barbican.plugin.crypto.p11_crypto.P11CryptoPlugin.'
                        '_create_pkcs11'), \
                mock.patch('kmip.pie.client.ProxyKmipClient'):

            secret_store.SecretStorePluginManager()


class WhenTestingSecretStores(utils.BarbicanAPIBaseTestCase,
                              SecretStoresMixin):

    def setUp(self):
        super(WhenTestingSecretStores, self).setUp()
        self.secret_store_repo = repos.get_secret_stores_repository()

    def test_should_get_all_secret_stores(self):

        g_index = 2  # global default index in plugins list
        self._init_multiple_backends(global_default_index=g_index)

        resp = self.app.get('/secret-stores', expect_errors=False)
        self.assertEqual(200, resp.status_int)
        secret_stores_data = resp.json.get('secret_stores')

        self.assertEqual(3, len(secret_stores_data))

        for i, secret_data in enumerate(secret_stores_data):
            self.assertEqual(i == g_index, secret_data['global_default'])
            self.assertIsNotNone(secret_data['secret_store_ref'])
            self.assertIsNone(secret_data.get('id'))
            self.assertIsNone(secret_data.get('secret_store_id'))
            self.assertIsNotNone(secret_data['name'])
            self.assertIsNotNone(secret_data['secret_store_plugin'])
            self.assertIsNotNone(secret_data['created'])
            self.assertIsNotNone(secret_data['updated'])
            self.assertEqual(models.States.ACTIVE, secret_data['status'])

    def test_get_all_secret_stores_when_multiple_backends_not_enabled(self):

        self._init_multiple_backends(enabled=False)

        resp = self.app.get('/secret-stores', expect_errors=True)
        self.assertEqual(404, resp.status_int)

        resp = self.app.get('/secret-stores/any_valid_id',
                            expect_errors=True)
        self.assertEqual(404, resp.status_int)

    def test_get_all_secret_stores_with_unsupported_http_method(self):

        self._init_multiple_backends()

        resp = self.app.put('/secret-stores', expect_errors=True)
        self.assertEqual(405, resp.status_int)

        resp = self.app.patch('/secret-stores', expect_errors=True)
        self.assertEqual(405, resp.status_int)

    def test_should_get_global_default(self):

        self._init_multiple_backends(global_default_index=1)

        resp = self.app.get('/secret-stores/global-default',
                            expect_errors=False)
        self.assertEqual(200, resp.status_int)
        resp_data = resp.json
        self.assertTrue(resp_data['global_default'])
        self.assertIn('kmip', resp_data['name'].lower())
        self.assertIsNotNone(resp_data['secret_store_ref'])
        self.assertIsNotNone(resp_data['secret_store_plugin'])
        self.assertIsNone(resp_data['crypto_plugin'])
        self.assertIsNotNone(resp_data['created'])
        self.assertIsNotNone(resp_data['updated'])
        self.assertEqual(models.States.ACTIVE, resp_data['status'])

    def test_get_global_default_when_multiple_backends_not_enabled(self):

        self._init_multiple_backends(enabled=False)

        with mock.patch('barbican.common.resources.'
                        'get_or_create_project') as m1:

            resp = self.app.get('/secret-stores/global-default',
                                expect_errors=True)

            self.assertFalse(m1.called)
            self.assertEqual(404, resp.status_int)

    def test_get_preferred_when_preferred_is_set(self):
        self._init_multiple_backends(global_default_index=1)

        secret_stores = self.secret_store_repo.get_all()
        project1 = self._create_project()

        self._create_project_store(project1.id, secret_stores[0].id)

        self.app.extra_environ = {
            'barbican.context': self._build_context(project1.external_id)
        }
        resp = self.app.get('/secret-stores/preferred',
                            expect_errors=False)
        self.assertEqual(200, resp.status_int)
        resp_data = resp.json
        self.assertEqual(secret_stores[0].name, resp_data['name'])
        self.assertEqual(secret_stores[0].global_default,
                         resp_data['global_default'])
        self.assertIn('/secret-stores/{0}'.format(secret_stores[0].id),
                      resp_data['secret_store_ref'])
        self.assertIsNotNone(resp_data['created'])
        self.assertIsNotNone(resp_data['updated'])
        self.assertEqual(models.States.ACTIVE, resp_data['status'])

    def test_get_preferred_when_preferred_is_not_set(self):
        self._init_multiple_backends(global_default_index=1)
        project1 = self._create_project()

        self.app.extra_environ = {
            'barbican.context': self._build_context(project1.external_id)
        }
        resp = self.app.get('/secret-stores/preferred',
                            expect_errors=True)
        self.assertEqual(404, resp.status_int)

    def test_get_preferred_when_multiple_backends_not_enabled(self):

        self._init_multiple_backends(enabled=False)

        with mock.patch('barbican.common.resources.'
                        'get_or_create_project') as m1:

            resp = self.app.get('/secret-stores/preferred',
                                expect_errors=True)

            self.assertFalse(m1.called)
            self.assertEqual(404, resp.status_int)


class WhenTestingSecretStore(utils.BarbicanAPIBaseTestCase,
                             SecretStoresMixin):

    def setUp(self):
        super(WhenTestingSecretStore, self).setUp()
        self.secret_store_repo = repos.get_secret_stores_repository()

    def test_get_a_secret_store_when_no_error(self):

        self._init_multiple_backends()

        secret_stores = self.secret_store_repo.get_all()

        store = secret_stores[0]

        resp = self.app.get('/secret-stores/{0}'.format(store.id),
                            expect_errors=False)
        self.assertEqual(200, resp.status_int)
        data = resp.json
        self.assertEqual(store.global_default, data['global_default'])
        self.assertEqual(store.name, data['name'])
        self.assertIn('/secret-stores/{0}'.format(store.id),
                      data['secret_store_ref'])
        self.assertIsNotNone(data['secret_store_plugin'])
        self.assertIsNotNone(data['created'])
        self.assertIsNotNone(data['updated'])
        self.assertEqual(models.States.ACTIVE, data['status'])

    def test_invalid_uri_for_secret_stores_subresource(self):
        self._init_multiple_backends()

        resp = self.app.get('/secret-stores/invalid_uri',
                            expect_errors=True)
        self.assertEqual(404, resp.status_int)

    def test_get_a_secret_store_with_unsupported_http_method(self):
        self._init_multiple_backends()

        secret_stores = self.secret_store_repo.get_all()
        store_id = secret_stores[0].id

        resp = self.app.put('/secret-stores/{0}'.format(store_id),
                            expect_errors=True)
        self.assertEqual(405, resp.status_int)

    def test_invalid_uri_for_a_secret_store_subresource(self):
        self._init_multiple_backends()

        secret_stores = self.secret_store_repo.get_all()
        resp = self.app.get('/secret-stores/{0}/invalid_uri'.
                            format(secret_stores[0].id), expect_errors=True)
        self.assertEqual(405, resp.status_int)


class WhenTestingProjectSecretStore(utils.BarbicanAPIBaseTestCase,
                                    SecretStoresMixin):

    def setUp(self):
        super(WhenTestingProjectSecretStore, self).setUp()
        self.secret_store_repo = repos.get_secret_stores_repository()
        self.proj_store_repo = repos.get_project_secret_store_repository()

    def test_set_a_preferred_secret_store_when_no_error(self):

        self._init_multiple_backends()

        stores = self.secret_store_repo.get_all()

        proj_external_id = uuidutils.generate_uuid(dashed=False)
        # get ids as secret store are not bound to session after a rest call.
        store_ids = [store.id for store in stores]
        for store_id in store_ids:
            self.app.extra_environ = {
                'barbican.context': self._build_context(proj_external_id)
            }
            resp = self.app.post('/secret-stores/{0}/preferred'.
                                 format(store_id), expect_errors=False)
            self.assertEqual(204, resp.status_int)

            # Now make sure preferred store is set to store id via get call
            resp = self.app.get('/secret-stores/preferred')
            self.assertIn(store_id, resp.json['secret_store_ref'])

    def test_unset_a_preferred_secret_store_when_no_error(self):

        self._init_multiple_backends()

        stores = self.secret_store_repo.get_all()

        proj_external_id = uuidutils.generate_uuid(dashed=False)
        # get ids as secret store are not bound to session after a rest call.
        store_ids = [store.id for store in stores]
        for store_id in store_ids:
            self.app.extra_environ = {
                'barbican.context': self._build_context(proj_external_id)
            }
            resp = self.app.post('/secret-stores/{0}/preferred'.
                                 format(store_id), expect_errors=False)
            self.assertEqual(204, resp.status_int)

            # unset preferred store here
            resp = self.app.delete('/secret-stores/{0}/preferred'.
                                   format(store_id), expect_errors=False)
            self.assertEqual(204, resp.status_int)

            # Now make sure that there is no longer a preferred store set
            resp = self.app.get('/secret-stores/preferred',
                                expect_errors=True)
            self.assertEqual(404, resp.status_int)

    def test_unset_a_preferred_store_when_not_found_error(self):
        self._init_multiple_backends()

        stores = self.secret_store_repo.get_all()
        proj_external_id = uuidutils.generate_uuid(dashed=False)
        self.app.extra_environ = {
            'barbican.context': self._build_context(proj_external_id)
        }
        resp = self.app.delete('/secret-stores/{0}/preferred'.
                               format(stores[0].id), expect_errors=True)
        self.assertEqual(404, resp.status_int)

    def test_preferred_secret_store_call_with_unsupported_http_method(self):
        self._init_multiple_backends()

        secret_stores = self.secret_store_repo.get_all()
        store_id = secret_stores[0].id

        proj_external_id = uuidutils.generate_uuid(dashed=False)

        self.app.extra_environ = {
            'barbican.context': self._build_context(proj_external_id)
        }
        resp = self.app.put('/secret-stores/{0}/preferred'.
                            format(store_id), expect_errors=True)

        self.assertEqual(405, resp.status_int)
